module params

  implicit none

  integer, parameter :: maxinv = 20         !max inventory bound
  integer, parameter:: numprods = 6   !number of brands

   character(63), parameter :: pathname = 'C:/Users/MattOsborne/Dropbox/price index paper/estimation data/'
  !character(59), parameter :: pathname = 'C:/Users/matthew/Dropbox/price index paper/estimation data/'

  integer, parameter :: nthread = 4   !number of threads

  character(4), parameter :: prodname = 'tuna'

  doubleprecision, allocatable :: sasprices(:,:,:)  !prices from SAS files
  integer stmarket(50)    !which market each store is in

  doubleprecision, allocatable :: bsavebig(:,:,:)        !individual draws on household parameters

  !supply side parameters
  integer, allocatable :: hhdatatemp(:,:,:)                  !tracks price states for simulated stuff
  integer, allocatable :: wtind(:,:)                         !state indexes for store-week

  doubleprecision, allocatable :: ain(:)        !parameter means
  doubleprecision, allocatable :: bin(:,:)   !individual level parmaters
  doubleprecision, allocatable :: din(:,:)  !parameter variances
  integer, allocatable :: scin(:)           !storage cost type (0 or 1)

  doubleprecision, allocatable :: aout(:)        !parameter means
  doubleprecision, allocatable :: bout(:,:)   !individual level parmaters
  doubleprecision, allocatable :: dout(:,:)  !parameter variances
  integer, allocatable :: scout(:)           !storage cost type (0 or 1)

  doubleprecision, allocatable :: verrorsav(:,:,:)    !draws on the consumption shock error
  doubleprecision, allocatable :: verrorsavold(:,:,:)    !old draws on consumption shock error
  integer, allocatable :: initialinv(:,:)           !draws on initial inventories

  doubleprecision, allocatable :: initprice(:,:,:)   !store prices before the data collection starts.
  doubleprecision, allocatable :: initweek(:,:,:)    !partially observed household data before store visits are collected
  integer, allocatable :: hhstorevisits(:,:,:)         !list of stores household visits
  integer, allocatable :: pstateptri(:,:)           !pointer mapping observed prices to save value functions - initial inv
  integer, allocatable :: pptrstatei(:,:)           !map from time/store pair to index for the price
  integer, allocatable :: visitsave(:,:)            !save store visit info - for when we don't want to redraw store visits
  integer, allocatable :: vlist(:,:)                !analog of hhweek(nhhvars-1,,) for initial periods.  Note that this can change as store draws change
  integer, allocatable :: vlisti(:,:)               !analog of hhweek(nhhvars,,) for initial periods.  Note that this can change as store draws change
  integer, allocatable :: lvlisti(:)                !number of initial price states stored

  doubleprecision, allocatable :: cstats(:,:,:,:,:,:)    ! consumption stats in vf

  integer, parameter :: allfixed = 0       !all coefficients fixed
  integer, parameter :: allrandom = 0      !all coefficients random

  doubleprecision, allocatable :: propv(:,:), propvi(:,:), propvic(:,:)  !fixed param proposals

  integer :: nhhs    !number of households in mcmc algorithm
  integer, parameter :: maxpurchases = 51   !total number of purchases
  integer, parameter :: wkstart = 72        !starting week for estimation

  !model parameters

  doubleprecision, parameter :: delta = 0.95               !discount factor
  doubleprecision, parameter :: pi = 3.141592654

  integer rseedsave(16)   !random seed for intrinsic random normal
  integer rseedstart(16)

  !estimation data - see my notes for definitions

  integer, parameter :: nhhvars = 26   !number of included variables in hhweek: 20 for 4 product model
  doubleprecision, allocatable :: hhweek(:,:,:)


  doubleprecision, allocatable :: hhprices(:,:)    !list of observed prices
  integer nobspr                                   !to save memory, we only calculate value function at observed prices

  integer, allocatable :: pstateptr(:,:)           !pointer mapping observed prices to save value functions
  integer, allocatable :: pptrstate(:,:)           !map from states to observed price points

  !structural model stuff

  integer, parameter :: nrpts = 25 !Number of interpolation points for prices
  integer nipts                      !number of inventory points
  integer, parameter :: nmdraws = 1     !draws on v when updating value function

  doubleprecision, parameter :: invdiv = 1.d0                     !number to divide inventories by

  integer, parameter :: nsavs = 30                 !number of saved previous value functions
  integer, parameter :: ntil = 1                  !number of value functions to use in nearest neighbor
  doubleprecision, allocatable :: vsav(:,:,:,:,:)
  doubleprecision, allocatable :: vsav1(:,:,:,:,:)       !used by getbell
  doubleprecision, allocatable :: vfsav(:,:)        !save value function for easy access in coupon loops
  integer(1), allocatable :: vfupd(:)               !tells us if we updated the value function last time - if so we draw more gammas
  doubleprecision, allocatable :: wtsavbig(:,:,:,:,:)      !importance weights for price interpolation
  doubleprecision, allocatable :: wtsavhhbig(:,:,:,:)
  doubleprecision, allocatable :: kernsavhh(:,:)      !individual kernels
  doubleprecision, allocatable :: vfupdtemp(:,:)      !value function for vfupdate, passed to optcons
  doubleprecision, allocatable :: avutilhh(:,:)        !flow utilities for price indexes
  doubleprecision, allocatable :: avutilhh2(:,:,:)        !flow utilities broken down across inclusive value and consumption
  doubleprecision, allocatable :: avutils(:)        !flow utilities for price indexes
  doubleprecision, allocatable :: avutilsav(:,:)    !saved flow utils
  doubleprecision, allocatable :: invsave(:,:)      !saved inventory
  doubleprecision, allocatable :: conssave(:,:)     !saved consumption


  ! hendel nevo variables
  doubleprecision, allocatable :: ivweek(:,:)        !inclusive value at each observation
  doubleprecision, allocatable :: ivsave(:,:)        !ivsave values indexed by value functions
  doubleprecision, allocatable :: ivdepgrid(:,:)   !grid points where we compute inclusive values - to make sure that we can get a decent fit
  doubleprecision, allocatable :: cutmat(:,:)     ! cut points for spline
  doubleprecision impdist(2)                      ! parameters of importance distribution
  doubleprecision, allocatable :: impdisthh(:,:)                      ! parameters of importance distribution - household specific
  integer, parameter :: ncutiv = 2                ! number of cut points in the inclusive value spline
  integer, allocatable :: hhcutiv(:)               ! for some individuals we may use a smaller grid
  doubleprecision, allocatable :: ivcoef(:,:)      ! inclusive value regression coefficients
  doubleprecision, allocatable :: ivsd(:)          !std dev of iv process
  logical, allocatable :: ivinclude(:,:)           ! tells us which coefficients we want to include.  For some individuals the coefficients are unstable so we should use a restricted spec
  integer, parameter :: vfitchange = 50            ! number of gibbs iterations where we do more value function iterations
  integer, parameter :: maxvfit = 11               ! number of vf iterations to do before gibbs iteration vfitchange

  ! these matter when we index all the price states - we shouldn't include variables we don't estimate
  logical, parameter :: usefeatures = .true.       !include features in state space
  logical, parameter :: usedisplay = .true.       !include display in state space

  integer iptimespred(54)    !predicted interpurchase timing

  doubleprecision, allocatable :: savedprices(:,:,:)   !saved price draws
  integer, allocatable :: savedpricesi(:,:,:)            !index of price draw (for discrete price importance dist)

  integer, parameter :: pdrawtype = 1                  ! 1 if we do discrete draws, 2 for continuous draws
  integer, allocatable :: primpdistf(:,:,:)            ! for pdrawtype 1 - indexes, frequencies of draws
  doubleprecision, allocatable :: primpdistp(:,:,:)    ! for pdrawtype 2 - probabilities of each index per household


  integer, parameter :: csetsize = 21            !size of choice set
  integer, parameter :: bigj = 10            !max number of packages you can buy - needed for normalization in hendel nevo model
  integer, allocatable :: choicesetmat(:,:)      !matrix containing all possible choices
  logical, parameter :: diagvar = .true.         !set this to true to estimate the inverse gamma model with diagonal variances
  logical, parameter :: sctypes = .false.         !if true, then estimate the model with heterogeneous storage cost types
  doubleprecision, parameter :: scwt = 0.d0         !weight on consumption in storage cost: 0 for begin of pd and 1 for end of pd
  logical, parameter :: twopoint = .true.        !two point model for storage costs
  integer, parameter :: nmulti = 2                 !number of storage cost types - set to 1 for the base model

  integer,allocatable :: iterarray(:)  !keeps track of how many different values we have stored for each hh
  doubleprecision,allocatable :: probsave(:)   !saved likelihoods
  integer itnum
  integer, allocatable :: acmat(:,:)   !value function index
  integer, allocatable ::acmatbig(:,:,:)    !bigger index - needed for parallel code to work

end module params



!work functions - random number generation, etc


subroutine DRNUN(n,x)

  !wrapper function for uniform rng

  use params
  implicit none

  integer n, status, i
  doubleprecision x(n)
  doubleprecision grnd

  do i=1,n
     x(i) = grnd()
  enddo

end subroutine DRNUN

subroutine DRNNOA(n,x)

  !wrapper for random normal rng

  use params
  implicit none

  integer n,status
  doubleprecision x(n)

  integer i
  doubleprecision random_normal

  do i=1,n
     x(i) = random_normal()
  enddo

end subroutine DRNNOA



subroutine DRNGAM(n,alpha,x)

  !wrapper for drngam
  !alpha is the shape parameter

  use params
  implicit none

  integer n
  doubleprecision alpha,x(n)
  integer status

  doubleprecision ran_gamma
  integer i

  do i=1,n
     x(i) = ran_gamma(alpha, i .eq. 1)
  enddo

end subroutine drngam



subroutine DRNBETA(n,alpha,beta,x)

  !wrapper for random beta rng

  use params
  implicit none

  integer n,status
  doubleprecision x(n), alpha, beta

  integer i
  doubleprecision random_beta

  do i=1,n
     x(i) = random_beta(alpha,beta,i .eq. 1)
  enddo

end subroutine DRNBETA


subroutine DRLSE(nipts1, Y, NIND, X, nipts2, con, B, SST, SSE)

  !wrapper for regression

  implicit none

  integer nipts1, nipts2, NIND, con

  doubleprecision Y(nipts1), x(nipts2,NIND), B(NIND+con)

  doubleprecision xnew(nipts2,NIND+con)

  doubleprecision xx(NIND+CON,NIND+CON), xy(NIND+CON,1)

  integer info, i

  doubleprecision work(1,nipts2), tau(nipts2)

  doubleprecision ynew(nipts1,1)

  doubleprecision ones(nipts1,1),yt(nipts1,1),yhat(nipts1,1),b1(NIND+CON,1)

  doubleprecision SST, SSE

  external dposv

  if(CON .eq. 1) then

     xnew(:,1) = 1.d0

     xnew(:,2:(NIND+1)) = x

  else

     xnew = x

  endif

  ynew(:,1) = Y

  xx = matmul(transpose(xnew),xnew)
  xy = matmul(transpose(xnew),ynew)

  call dposv('U',NIND+CON,1,xx,NIND+CON,xy,NIND+CON,info) !this is a lapack routine.  Get it and dependencies at http://www.netlib.org/lapack/double

  B = xy(:,1)

  SSE = 0.d0
  SST = 1.d0

end subroutine DRLSE


subroutine invertmat(X,XI,N)

  ! invert positive definite matrix - from netlib

  implicit none

  integer N
  doubleprecision X(N,N), XI(N,N), EYE(N,N)

  integer i, info, j
  external dposv

  EYE = 0.d0
  do i=1,N
     EYE(i,i) = 1.d0
  enddo

  call dposv('U',N,N,X,N,EYE,N,info)
  XI = EYE

  ! this replicates R's functionality
  do i=1,N
     do j=i+1,N
        XI(j,i) = XI(i,j)
     enddo
  enddo

  if(info .lt. 0) then
     print*,'Error in dposv: illegal value in column',abs(info)
     stop
  else if (info .gt. 0) then
     print*,'Error in dposv: the principal minor of order ',info,' is not positive definite.'
     stop
  endif

end subroutine invertmat


subroutine chol(X,XC,N)

  ! cholesky factorize positive definite matrix

  implicit none

  integer N
  doubleprecision X(N,N), XC(N,N)

  integer info, i, j

  external dpotrf

  XC = X

  call dpotrf( 'U', N, XC, N, info )

  do i=1,N
     do j=i+1,N
        XC(j,i) = 0.d0
     enddo
  enddo

  if(info .lt. 0) then
     print*,'Error in dpotrf: illegal value in column',abs(info)
     stop
  else if (info .gt. 0) then
     print*,'Error in dpotrf: the principal minor of order ',info,' is not positive definite.'
     stop
  endif


end subroutine chol


doubleprecision function stddev(x,n)

  implicit none

  integer n
  doubleprecision x(n), ex, ex2, dn

  integer i

  stddev = 0.d0
  ex = 0.d0
  ex2 = 0.d0

  dn = dble(n)

  do i=1,n

     ex = ex + x(i)
     ex2 = ex2 + x(i)*x(i)

  enddo

  stddev = sqrt( (dn*ex2 - ex*ex)/(dn*(dn-1)) )

end function stddev


! quick function to compute variance matrix

subroutine varmat(x,n1,n2,v)

  implicit none

  integer n1,n2
  doubleprecision x(n1,n2), v(n1,n1)

  doubleprecision m(n1)

  integer i,j

  m = 0.d0

  do i=1,n2
     do j=1,n1
        m(j) = m(j) + x(j,i)
     enddo
  enddo

  m=m/dble(n2)

  v = 0.d0

  do i=1,n1
     do j=1,n1
        v(i,j) = dot_product(x(i,:)-m(i),x(j,:)-m(j))
     enddo
  enddo
  v=v/dble(n2)

end subroutine varmat

!OTHER PROGRAM FUNCTIONS


integer function toint(charray)

  !func to change string to integer

  implicit none
  character(*) charray
  integer i

  toint = 0

  do i=1,len(charray)

     toint = toint + (ichar(charray(len(charray)-i+1:len(charray)-i+1)) - 48)*(10**(i-1))

  end do

end function toint


! round to nearest cent

doubleprecision function round2(x)

  doubleprecision x

  round2 = dble( int( x*100.d0 + 0.5d0 ) )/100.d0

end function round2


doubleprecision function logistic(mu)

  !logistic random number generator

  use params
  implicit none

  doubleprecision x(1), mu

  external DRNUN

  call DRNUN(1,x(1))

  logistic = -mu*log(1/x(1) - 1)

end function logistic




doubleprecision function logitfn(y)

  !logistic cdf

  use params
  implicit none

  doubleprecision x(1),y

  external DRNUN

  call DRNUN(1,x(1))

  logitfn = -log(-log(x(1)))

end function logitfn



doubleprecision function normpdf(x,mu,sig)

  !function for normal pdf

  use params
  implicit none

  doubleprecision x,mu,sig


  normpdf = (1.d0/(sqrt(2.d0*pi)*sig))*exp(-0.5d0*((x-mu)/sig)**2 )

end function normpdf




logical function inlist(i,n,x)

  !see if i is in the list x

  implicit none

  integer i,n,x(n)

  integer j

  inlist = .false.

  do j=1,n

    inlist = inlist .or. (x(j) .eq. i)

  enddo

end function inlist

! find index of element in list - return -1 if nothing is found

integer function findinlist(i,n,x)

  !see if i is in the list x

  implicit none

  integer i,n,x(n)

  integer j

  findinlist = -1

  do j=1,n

    if(x(j) .eq. i) findinlist = j

  enddo

end function findinlist




character(3) function ccond(c)

  implicit none

  logical c

  if(c) then
    ccond='YES'
  else
    ccond='NO'
  endif

end function ccond


doubleprecision function dcond(x,c)

  !return x if c is true, 0 otherwise

  implicit none
  logical c
  doubleprecision x

  if(c) then
    dcond=x
  else
    dcond=0.d0
  endif

end function dcond







character function toch(i)

  !turn integer to character

  integer i

  toch = char(48+i)

end function toch


doubleprecision function dmax(a,b)


  doubleprecision a,b

  if(a .gt. b) then
    dmax = a
  else
    dmax = b
  endif

end function dmax


doubleprecision function dmin(a,b)


  doubleprecision a,b

  if(a .lt. b) then
    dmin = a
  else
    dmin = b
  endif

end function dmin


! vector min and max

subroutine pmin(a,n,mina,dmina)

  implicit none

  integer n, dmina

  doubleprecision a(n), mina

  integer i

  mina = a(1)

  dmina = 1

  do i=2,n

     if(a(i) < mina) then
        mina = a(i)
        dmina = i
     endif

  enddo


end subroutine pmin


subroutine pmax(a,n,maxa,dmaxa)

  implicit none

  integer n, dmaxa

  doubleprecision a(n), maxa

  integer i

  maxa = a(1)

  dmaxa = 1

  do i=2,n

     if(a(i) > maxa) then
        maxa = a(i)
        dmaxa = i
     endif

  enddo


end subroutine pmax


! b spline basis function

doubleprecision function phifn(t)

  implicit none

  doubleprecision t, at

  at = abs(t)

  if(at > 1.d0 .and. at <= 2.d0) then
    phifn = (2.0-at)*(2.0-at)*(2.0-at)
  else if (at <= 1.d0) then
    phifn = 4.0-6.0*at*at+3.0*at*at*at
  else
    phifn = 0.d0
  endif

end function phifn


! compute b spline

subroutine bsplinei(x, cuts, lcuts, y)

  implicit none

  integer lcuts

  doubleprecision x, cuts(2), y(lcuts+2)

  doubleprecision h, phifn
  integer n,i

  n = lcuts - 1
  h = cuts(2)-cuts(1)

  do i=1,n+3
     y(i) = phifn( (x-cuts(1))/h - ( dble(i) - 2.d0 ) )
  enddo

end subroutine bsplinei


! check if all arguments of an array are zero

logical function allzero(x,n)

  implicit none
  integer n
  doubleprecision x(n)

  integer i

  allzero = .true.

  do i=1,n

     allzero = allzero .and. x(i) .eq. 0.d0

  enddo

end function allzero



doubleprecision FUNCTION random_normal() ! RESULT(fn_val)

	! Adapted from the following Fortran 77 code
	!      ALGORITHM 712, COLLECTED ALGORITHMS FROM ACM.
	!      THIS WORK PUBLISHED IN TRANSACTIONS ON MATHEMATICAL SOFTWARE,
	!      VOL. 18, NO. 4, DECEMBER, 1992, PP. 434-435.

	!  The function random_normal() returns a normally distributed pseudo-random
	!  number with zero mean and unit variance.

	!  The algorithm uses the ratio of uniforms method of A.J. Kinderman
	!  and J.F. Monahan augmented with quadratic bounding curves.

	!doubleprecision :: fn_val

	!     Local variables
	doubleprecision     :: s = 0.449871, t = -0.386595, a = 0.19600, b = 0.25472,           &
	            r1 = 0.27597, r2 = 0.27846, u, v, x, y, q

 doubleprecision grnd

	!     Generate P = (u,v) uniform in rectangle enclosing acceptance region

	DO
          u = grnd()
          v = grnd()
	  v = 1.7156 * (v - 0.5d0)
	!     Evaluate the quadratic form
	  x = u - s
	  y = ABS(v) - t
	  q = x**2 + y*(a*y - b*x)
	!     Accept P if inside inner ellipse
	  IF (q < r1) EXIT
	!     Reject P if outside outer ellipse
	  IF (q > r2) CYCLE
	!     Reject P if outside acceptance region
	  IF (v**2 < -4.0*LOG(u)*u**2) EXIT
	END DO

	!     Return ratio of P's coordinates as the normal deviate
	random_normal = v/u
	RETURN

END FUNCTION random_normal


doubleprecision FUNCTION ran_gamma(s, first) !RESULT(fn_val)

	IMPLICIT NONE

	! Uses the algorithm in
	! Marsaglia, G. and Tsang, W.W. (2000) `A simple method for generating
	! gamma variables', Trans. om Math. Software (TOMS).

	! Generates a random gamma deviate for shape parameter s >= 1.

	doubleprecision, INTENT(IN)    :: s
	LOGICAL, INTENT(IN) :: first
	doubleprecision                :: fn_val
	doubleprecision random_normal

	! Local variables
	doubleprecision, SAVE  :: c, d
	doubleprecision        :: u, v, x

	IF (s < 1.0) THEN
	   WRITE(*, *) 'Shape parameter must be >= 1',s
	   STOP
	END IF

	IF (first) THEN
	  d = s - 1./3.
	  c = 1.0/SQRT(9.0*d)
	END IF

	! Start of main loop
	DO

	! Generate v = (1+cx)^3 where x is random normal; repeat if v <= 0.

	  DO
	    x = random_normal()
	    v = (1.0 + c*x)**3
	    IF (v > 0.0) EXIT
	  END DO

	! Generate uniform variable U

	  CALL RANDOM_NUMBER(u)
	  IF (u < 1.0 - 0.0331*x**4) THEN
	    ran_gamma = d*v
	    EXIT
	  ELSE IF (LOG(u) < 0.5*x**2 + d*(1.0 - v + LOG(v))) THEN
	    ran_gamma = d*v
	    EXIT
	  END IF
	END DO

	RETURN
END FUNCTION ran_gamma



doubleprecision FUNCTION random_gamma(s, first) !RESULT(fn_val)

	! Adapted from Fortran 77 code from the book:
	!     Dagpunar, J. 'Principles of random variate generation'
	!     Clarendon Press, Oxford, 1988.   ISBN 0-19-852202-9

	!     FUNCTION GENERATES A RANDOM GAMMA VARIATE.
	!     CALLS EITHER random_gamma1 (S > 1.0)
	!     OR random_exponential (S = 1.0)
	!     OR random_gamma2 (S < 1.0).

	!     S = SHAPE PARAMETER OF DISTRIBUTION (0 < REAL).

	doubleprecision, INTENT(IN)    :: s
	LOGICAL, INTENT(IN) :: first
	doubleprecision random_gamma1,random_gamma2,random_exponential

	IF (s <= zero) THEN
	  WRITE(*, *) 'SHAPE PARAMETER VALUE MUST BE POSITIVE'
	  STOP
	END IF

	IF (s > one) THEN
	  random_gamma = random_gamma1(s, first)
	ELSE IF (s < one) THEN
	  random_gamma = random_gamma2(s, first)
	ELSE
	  random_gamma = random_exponential()
	END IF

	RETURN
END FUNCTION random_gamma



doubleprecision FUNCTION random_gamma1(s, first) !RESULT(fn_val)

	! Uses the algorithm in
	! Marsaglia, G. and Tsang, W.W. (2000) `A simple method for generating
	! gamma variables', Trans. om Math. Software (TOMS), vol.26(3), pp.363-372.

	! Generates a random gamma deviate for shape parameter s >= 1.

	doubleprecision, INTENT(IN)    :: s
	LOGICAL, INTENT(IN) :: first
	doubleprecision random_normal, grnd

	! Local variables
	doubleprecision, SAVE  :: c, d
	doubleprecision        :: u, v, x

	IF (first) THEN
	  d = s - one/3.
	  c = one/SQRT(9.0*d)
	END IF

	! Start of main loop
	DO

	! Generate v = (1+cx)^3 where x is random normal; repeat if v <= 0.

	  DO
	    x = random_normal()
	    v = (one + c*x)**3
	    IF (v > zero) EXIT
	  END DO

	! Generate uniform variable U

	  !CALL RANDOM_NUMBER(u)
	  u = grnd()
	  IF (u < one - 0.0331*x**4) THEN
	    random_gamma1 = d*v
	    EXIT
	  ELSE IF (LOG(u) < 0.5d0*x**2 + d*(one - v + LOG(v))) THEN
	    random_gamma1 = d*v
	    EXIT
	  END IF
	END DO

	RETURN
END FUNCTION random_gamma1



doubleprecision FUNCTION random_gamma2(s, first) !RESULT(fn_val)

	! Adapted from Fortran 77 code from the book:
	!     Dagpunar, J. 'Principles of random variate generation'
	!     Clarendon Press, Oxford, 1988.   ISBN 0-19-852202-9

	! FUNCTION GENERATES A RANDOM VARIATE IN [0,INFINITY) FROM
	! A GAMMA DISTRIBUTION WITH DENSITY PROPORTIONAL TO
	! GAMMA2**(S-1) * EXP(-GAMMA2),
	! USING A SWITCHING METHOD.

	!    S = SHAPE PARAMETER OF DISTRIBUTION
	!          (REAL < 1.0)

	doubleprecision, INTENT(IN)    :: s
	LOGICAL, INTENT(IN) :: first
	doubleprecision                :: fn_val

	!     Local variables
	doubleprecision       :: r, x, w
	doubleprecision, SAVE :: a, p, c, uf, vr, d
	doubleprecision grnd

	IF (s <= zero .OR. s >= one) THEN
	  WRITE(*, *) 'SHAPE PARAMETER VALUE OUTSIDE PERMITTED RANGE'
	  STOP
	END IF

	IF (first) THEN                        ! Initialization, if necessary
	  a = one - s
	  p = a/(a + s*EXP(-a))
	  IF (s < vsmall) THEN
	    WRITE(*, *) 'SHAPE PARAMETER VALUE TOO SMALL'
	    STOP
	  END IF
	  c = one/s
	  uf = p*(vsmall/a)**s
	  vr = one - vsmall
	  d = a*LOG(a)
	END IF

	DO
	  !CALL RANDOM_NUMBER(r)
	  r = grnd()
	  IF (r >= vr) THEN
	    CYCLE
	  ELSE IF (r > p) THEN
	    x = a - LOG((one - r)/(one - p))
	    w = a*LOG(x)-d
	  ELSE IF (r > uf) THEN
	    x = a*(r/p)**c
	    w = x
	  ELSE
	    random_gamma2 = zero
	    RETURN
	  END IF

	  !CALL RANDOM_NUMBER(r)
	  r = grnd()
	  IF (one-r <= w .AND. r > zero) THEN
	    IF (r*(w + one) >= one) CYCLE
	    IF (-LOG(r) <= w) CYCLE
	  END IF
	  EXIT
	END DO

	random_gamma2 = x
	RETURN

END FUNCTION random_gamma2



doubleprecision FUNCTION random_exponential() !RESULT(fn_val)

	! Adapted from Fortran 77 code from the book:
	!     Dagpunar, J. 'Principles of random variate generation'
	!     Clarendon Press, Oxford, 1988.   ISBN 0-19-852202-9

	! FUNCTION GENERATES A RANDOM VARIATE IN [0,INFINITY) FROM
	! A NEGATIVE EXPONENTIAL DlSTRIBUTION WlTH DENSITY PROPORTIONAL
	! TO EXP(-random_exponential), USING INVERSION.

	doubleprecision  :: fn_val

	!     Local variable
	doubleprecision  :: r, grnd

	DO
	  !CALL RANDOM_NUMBER(r)
	  r = grnd()
	  IF (r > zero) EXIT
	END DO

	random_exponential = -LOG(r)
	RETURN

END FUNCTION random_exponential



doubleprecision FUNCTION random_beta(aa, bb, first) !RESULT(fn_val)

  ! Adapted from Fortran 77 code from the book:
  !     Dagpunar, J. 'Principles of random variate generation'
  !     Clarendon Press, Oxford, 1988.   ISBN 0-19-852202-9

  ! FUNCTION GENERATES A RANDOM VARIATE IN [0,1]
  ! FROM A BETA DISTRIBUTION WITH DENSITY
  ! PROPORTIONAL TO BETA**(AA-1) * (1-BETA)**(BB-1).
  ! USING CHENG'S LOG LOGISTIC METHOD.

  !     AA = SHAPE PARAMETER FROM DISTRIBUTION (0 < REAL)
  !     BB = SHAPE PARAMETER FROM DISTRIBUTION (0 < REAL)

  implicit none

  doubleprecision, INTENT(IN)    :: aa, bb
  LOGICAL, INTENT(IN) :: first
  doubleprecision                :: fn_val

  !     Local variables
  doubleprecision, PARAMETER  :: aln4 = 1.3862944
  doubleprecision             :: a, b, g, r, s, x, y, z
  doubleprecision, SAVE       :: d, f, h, t, c
  LOGICAL, SAVE    :: swap

  doubleprecision, parameter :: zero = 0.d0
  doubleprecision, parameter :: one = 1.d0
  doubleprecision, parameter :: two = 2.d0
  doubleprecision, parameter :: vlarge = 1e10
  doubleprecision, parameter :: vsmall = 1e-10
  doubleprecision grnd

  IF (aa <= zero .OR. bb <= zero) THEN
     WRITE(*, *) 'IMPERMISSIBLE SHAPE PARAMETER VALUE(S)'
     STOP
  END IF

  IF (first) THEN                        ! Initialization, if necessary
     a = aa
     b = bb
     swap = b > a
     IF (swap) THEN
        g = b
        b = a
        a = g
     END IF
     d = a/b
     f = a+b
     IF (b > one) THEN
        h = SQRT((two*a*b - f)/(f - two))
        t = one
     ELSE
        h = b
        t = one/(one + (a/(vlarge*b))**b)
     END IF
     c = a+h
  END IF

  DO
     r = grnd()
     x = grnd()
     s = r*r*x
     IF (r < vsmall .OR. s <= zero) CYCLE
     IF (r < t) THEN
        x = LOG(r/(one - r))/h
        y = d*EXP(x)
        z = c*x + f*LOG((one + d)/(one + y)) - aln4
        IF (s - one > z) THEN
           IF (s - s*z > one) CYCLE
           IF (LOG(s) > z) CYCLE
        END IF
        fn_val = y/(one + y)
     ELSE
        IF (4.0*s > (one + one/d)**f) CYCLE
        fn_val = one
     END IF
     EXIT
  END DO

  IF (swap) fn_val = one - fn_val
  random_beta = fn_val
  RETURN
END FUNCTION random_beta



      DOUBLE PRECISION FUNCTION PHID(Z)
!
!     Normal distribution probabilities accurate to 1d-15.
!     Reference: J.L. Schonfelder, Math Comp 32(1978), pp 1232-1240.
!
      INTEGER I, IM
      DOUBLE PRECISION A(0:43), BM, B, BP, P, RTWO, T, XA, Z
      PARAMETER( RTWO = 1.414213562373095048801688724209D0, IM = 24 )
      SAVE A
      DATA ( A(I), I = 0, 43 )/&
         6.10143081923200417926465815756D-1,&
        -4.34841272712577471828182820888D-1,&
         1.76351193643605501125840298123D-1,&
        -6.0710795609249414860051215825D-2,&
         1.7712068995694114486147141191D-2,&
        -4.321119385567293818599864968D-3, &
         8.54216676887098678819832055D-4, &
        -1.27155090609162742628893940D-4,&
         1.1248167243671189468847072D-5, 3.13063885421820972630152D-7,      &
        -2.70988068537762022009086D-7, 3.0737622701407688440959D-8,&
         2.515620384817622937314D-9, -1.028929921320319127590D-9,&
         2.9944052119949939363D-11, 2.6051789687266936290D-11,&
        -2.634839924171969386D-12, -6.43404509890636443D-13,&
         1.12457401801663447D-13, 1.7281533389986098D-14, &
        -4.264101694942375D-15, -5.45371977880191D-16,&
         1.58697607761671D-16, 2.0899837844334D-17, &
        -5.900526869409D-18, -9.41893387554D-19, 2.14977356470D-19, &
         4.6660985008D-20, -7.243011862D-21, -2.387966824D-21, &
         1.91177535D-22, 1.20482568D-22, -6.72377D-25, -5.747997D-24,&
        -4.28493D-25, 2.44856D-25, 4.3793D-26, -8.151D-27, -3.089D-27, &
         9.3D-29, 1.74D-28, 1.6D-29, -8.0D-30, -2.0D-30 /
!
      XA = ABS(Z)/RTWO
      IF ( XA .GT. 100 ) THEN
         P = 0
      ELSE
         T = ( 8*XA - 30 ) / ( 4*XA + 15 )
         BM = 0
         B  = 0
         DO I = IM, 0, -1
            BP = B
            B  = BM
            BM = T*B - BP  + A(I)
         END DO
         P = EXP( -XA*XA )*( BM - BP )/4
      END IF
      IF ( Z .GT. 0 ) P = 1 - P
      PHID = P
      END

! ren-raw chen, rutgers business school
! normal inverse
! translate from
! http://home.online.no/~pjacklam/notes/invnorm
! a routine written by john herrero
      real*8 function dinvnorm(p)
      real*8 p,p_low,p_high
      real*8 a1,a2,a3,a4,a5,a6
      real*8 b1,b2,b3,b4,b5
      real*8 c1,c2,c3,c4,c5,c6
      real*8 d1,d2,d3,d4
      real*8 z,q,r
      a1=-39.6968302866538
      a2=220.946098424521
      a3=-275.928510446969
      a4=138.357751867269
      a5=-30.6647980661472
      a6=2.50662827745924
      b1=-54.4760987982241
      b2=161.585836858041
      b3=-155.698979859887
      b4=66.8013118877197
      b5=-13.2806815528857
      c1=-0.00778489400243029
      c2=-0.322396458041136
      c3=-2.40075827716184
      c4=-2.54973253934373
      c5=4.37466414146497
      c6=2.93816398269878
      d1=0.00778469570904146
      d2=0.32246712907004
      d3=2.445134137143
      d4=3.75440866190742
      p_low=0.02425
      p_high=1-p_low
      if(p.lt.p_low) goto 201
      if(p.ge.p_low) goto 301
201   q=dsqrt(-2*dlog(p))
      z=(((((c1*q+c2)*q+c3)*q+c4)*q+c5)*q+c6)/((((d1*q+d2)*q+d3)*q+d4)*q+1)
      goto 204
301   if((p.ge.p_low).and.(p.le.p_high)) goto 202
      if(p.gt.p_high) goto 302
202   q=p-0.5
      r=q*q
      z=(((((a1*r+a2)*r+a3)*r+a4)*r+a5)*r+a6)*q/(((((b1*r+b2)*r+b3)*r+b4)*r+b5)*r+1)
      goto 204
302   if((p.gt.p_high).and.(p.lt.1)) goto 203
203   q=dsqrt(-2*dlog(1-p))
      z=-(((((c1*q+c2)*q+c3)*q+c4)*q+c5)*q+c6)/((((d1*q+d2)*q+d3)*q+d4)*q+1)
204   dinvnorm=z
      return
      end



      subroutine sgrnd(seed)
!
      implicit integer(a-z)
!
! Period parameters
      parameter(N     =  624)
!
      dimension mt(0:N-1)
!                     the array for the state vector
      common /block/mti,mt
      save   /block/
!
!      setting initial seeds to mt[N] using
!      the generator Line 25 of Table 1 in
!      [KNUTH 1981, The Art of Computer Programming
!         Vol. 2 (2nd Ed.), pp102]
!
      mt(0)= iand(seed,-1)
      do 1000 mti=1,N-1
        mt(mti) = iand(69069 * mt(mti-1),-1)
 1000 continue
!
      return
      end
!***********************************************************************
      double precision function grnd()
!
      implicit integer(a-z)
!
! Period parameters
      parameter(N     =  624)
      parameter(N1    =  N+1)
      parameter(M     =  397)
      parameter(MATA  = -1727483681)
!                                    constant vector a
      parameter(UMASK = -2147483648)
!                                    most significant w-r bits
      parameter(LMASK =  2147483647)
!                                    least significant r bits
! Tempering parameters
      parameter(TMASKB= -1658038656)
      parameter(TMASKC= -272236544)
!
      dimension mt(0:N-1)
!                     the array for the state vector
      common /block/mti,mt
      save   /block/
      data   mti/N1/
!                     mti==N+1 means mt[N] is not initialized
!
      dimension mag01(0:1)
      data mag01/0, MATA/
      save mag01
!                        mag01(x) = x * MATA for x=0,1
!
      TSHFTU(y)=ishft(y,-11)
      TSHFTS(y)=ishft(y,7)
      TSHFTT(y)=ishft(y,15)
      TSHFTL(y)=ishft(y,-18)
!
      if(mti.ge.N) then
!                       generate N words at one time
        if(mti.eq.N+1) then
!                            if sgrnd() has not been called,
          call sgrnd(4357)
!                              a default initial seed is used
        endif
!
        do 1000 kk=0,N-M-1
            y=ior(iand(mt(kk),UMASK),iand(mt(kk+1),LMASK))
            mt(kk)=ieor(ieor(mt(kk+M),ishft(y,-1)),mag01(iand(y,1)))
 1000   continue
        do 1100 kk=N-M,N-2
            y=ior(iand(mt(kk),UMASK),iand(mt(kk+1),LMASK))
            mt(kk)=ieor(ieor(mt(kk+(M-N)),ishft(y,-1)),mag01(iand(y,1)))
 1100   continue
        y=ior(iand(mt(N-1),UMASK),iand(mt(0),LMASK))
        mt(N-1)=ieor(ieor(mt(M-1),ishft(y,-1)),mag01(iand(y,1)))
        mti = 0
      endif
!
      y=mt(mti)
      mti=mti+1
      y=ieor(y,TSHFTU(y))
      y=ieor(y,iand(TSHFTS(y),TMASKB))
      y=ieor(y,iand(TSHFTT(y),TMASKC))
      y=ieor(y,TSHFTL(y))
!
      if(y.lt.0) then
        grnd=(dble(y)+2.0d0**32)/(2.0d0**32-1.0d0)
      else
        grnd=dble(y)/(2.0d0**32-1.0d0)
      endif
!
      return
      end
