# look at sale frequencies, etc, across the markets and over time to see if we can get anything

rm(list=ls())

product = "sssoup"
inputpath = "C:/Users/MattOsborne/Dropbox/price index paper/input data/"                   #raw data
outputpath = "C:/Users/MattOsborne/Dropbox/price index paper/step 1 output files/"         #intermediate output
tablepath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"    #tables for paper
figpath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"      #tables for paper

load(paste(outputpath,"erim_",product,".RData",sep=""))

# make a small fix so flavors don't get mixed up

branddata$desc[branddata$upc == 5100001021] = "CAMP CD VEG SP                "

# convert price to cents
stdata$price = stdata$price/100

allstids = unique(stdata$storeid)
allstids = allstids[order(allstids)]

brandnames = c("CAMP","CTL BR","SWNSN")

flavor = c("CRM OF MSH","CD TOM","CH NDL","CRM OF CH","CRM OF CELRY","VEG BF","BN/BCN","CHEDDAR CHEESE","CLAM CHOW",
           "CH/RC","CRM OF POT","HMSTY CH NDL","VEGETARIAN VEG","CK O-F BN/HM","CH BR","CD VEG SP","CH GMB",
           "SPLT PEA/HM & BCN")

bdatabig = branddata
bdatabig$brandid = 0

j=0
for(i in brandnames) {
  j=j+1
  bdatabig$brandid[grep(i,bdatabig$desc)] = j
}

bdatabig$flavor = 0

j=0
for(i in flavor) {
  j=j+1
  bdatabig$flavor[grep(i,bdatabig$desc)] = j
}

# remove ready mix packets

packet = union(grep("I-MX",bdatabig$desc) , grep("R-MX",bdatabig$desc))

bdatabig$packet = 0
bdatabig$packet[packet] = 1

bdatabig$wtcode <- NULL
bdatabig$wtdesc <- NULL
bdatabig$eqfactor <- NULL
bdatabig$rsln <- NULL

# merge brand info data into purchase data

hhpurch$filler <- NULL

hhpurch = merge(hhpurch,bdatabig,all.x=TRUE)
hhpurch$brandname = "unassigned"
hhpurch$brandname[hhpurch$brandid>0] = brandnames[hhpurch$brandid[hhpurch$brandid > 0]]

hhpurch$uprice = hhpurch$price/pmax(100,hhpurch$units)  # 1 observation that is 0

hhpurch = hhpurch[hhpurch$packet == 0,]

# print out summary stats of household purchase data

cat("Number of households in Sioux Falls:", length(unique(hhpurch$hhid[hhpurch$market==1 & hhpurch$packet == 0])), "\n")
cat("Number of households in Springfield:", length(unique(hhpurch$hhid[hhpurch$market==2 & hhpurch$packet == 0])), "\n")

cat("Number of purchases in Sioux Falls:", length(hhpurch$hhid[hhpurch$market==1 & hhpurch$packet == 0]), "\n")
cat("Number of purchases in Springfield:", length(hhpurch$hhid[hhpurch$market==2 & hhpurch$packet == 0]), "\n")

# make table of market shares and size shares


bnamest = c("Campbell's","Store Brand","Swanson")

mktshares = matrix(0,nrow=length(bnamest),ncol=3)

for(i in 1:length(brandnames)) {
    for(j in 1:2) {
        if(i <= length(bnamest)) {
            mktshares[i,j] = sum(hhpurch$brandid == i & hhpurch$market == j)/sum(hhpurch$market==j)
        } else {
            mktshares[length(bnamest)-1,j] = mktshares[length(bnamest)-1,j] + sum(hhpurch$brandid == i & hhpurch$market == j)/sum(hhpurch$market==j)
        }
    }
}

for(i in 1:length(bnamest)) {
    mktshares[i,3] = (mktshares[i,1]*sum(hhpurch$market==1)+mktshares[i,2]*sum(hhpurch$market==2))/nrow(hhpurch)
}

o = order(mktshares[,3],decreasing=TRUE)
bnamest = bnamest[o]
mktshares = mktshares[o,]

sharetable = bnamest[mktshares[,3] > 0.01]

sigfig = 1

for(i in 1:length(sharetable)) {
    sharetable[i] = paste(sharetable[i]," & ",round(100*mktshares[i,1],sigfig)," \\\\",sep="")
}

write.table(sharetable,file=paste(tablepath,"share",product,".tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

print(table(hhpurch$weight)/nrow(hhpurch))

allweek = unique(hhpurch$week)
allweek = allweek[order(allweek)]

stdata = merge(stdata,bdatabig,all.x=TRUE)
stdata = stdata[order(stdata$storeid,stdata$week,stdata$brandid,stdata$flavor,stdata$upc),]

stprices = array(0,c(length(allstids),length(allweek),length(brandnames),2,2))

# remove purchases that are not in the categories used for estimation
# Additionally taking out Campbell's special request chicken noodle (51000058970, 359 purchases), sp rq cream of mushroom (286 purch)
# CAMP CK RTS CK CH NDL/MSH (1190)

badupc = c(51000058970,51000060070,51000005720)

badpurch = hhpurch$brandid == 0 | hhpurch$flavor == 0 | hhpurch$weight < 10500 | hhpurch$weight > 11500 | hhpurch$packet == 1 | hhpurch$upc %in% badupc

hhpurch = hhpurch[!badpurch,]

badst = stdata$brandid == 0 | stdata$flavor == 0 | stdata$wtamt < 10500 | stdata$wtamt > 11500 | stdata$packet == 1 | stdata$upc %in% badupc

stdata = stdata[!badst,]

hhpurch = hhpurch[order(hhpurch$hhid,hhpurch$week,hhpurch$brandid,hhpurch$flavor,hhpurch$day),]

# load week date translation

w2d = read.csv(paste(inputpath,"dateweektranslation.csv",sep=""),stringsAsFactors=FALSE)
w2d$mindate = as.Date(w2d$mindate,format="%m/%d/%Y")
w2d$maxdate = as.Date(w2d$maxdate,format="%m/%d/%Y")

# load the SAS price files

pf1 = read.csv(paste(inputpath,product,"/pf1.csv",sep=""),stringsAsFactors=FALSE)
pf2 = read.csv(paste(inputpath,product,"/pf2.csv",sep=""),stringsAsFactors=FALSE)

pf1$DATE = as.Date(pf1$DATE,origin="1960-01-01")
pf1$erimweek = 0
for(i in 1:nrow(w2d)) {
  pf1$erimweek[pf1$DATE >= w2d$mindate[i] & pf1$DATE <= w2d$maxdate[i]] = w2d$erimweek[i]
}

pf1 = merge(pf1,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")
save(pf1,file=paste(outputpath,"dailyprices1_",product,".RData",sep=""))

pf1 = pf1[pf1$brandid > 0 & pf1$wtamt >= 10500 & pf1$wtamt <= 11500,]

pf2$DATE = as.Date(pf2$DATE,origin="1960-01-01")

pf2$erimweek = 0
for(i in 1:nrow(w2d)) {
  pf2$erimweek[pf2$DATE >= w2d$mindate[i] & pf2$DATE <= w2d$maxdate[i]] = w2d$erimweek[i]
}

pf2 = merge(pf2,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")

pf2 = pf2[pf2$brandid > 0 & pf2$wtamt >= 10500 & pf2$wtamt <= 11500,]

# aggregate to the weekly level

pf1w = aggregate(pf1$DPR,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=min)

colnames(pf1w) = c("SID","erimweek","UPC","price")

uw = aggregate(pf1$UNITS,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=sum)

colnames(uw) = c("SID","erimweek","UPC","UNITS")

pf1w$UNITS = uw$UNITS

rp = aggregate(pf1$REG,by=list(pf1$SID,pf1$erimweek,pf1$UPC),FUN=min)

colnames(rp) = c("SID","erimweek","UPC","REG")

pf1w$REG = rp$REG

pf1w = merge(pf1w,bdatabig,all.x=TRUE,by.x="UPC",by.y="upc")

pf1w = pf1w[order(pf1w$SID,pf1w$UPC,pf1w$erimweek),]


pf2w = aggregate(pf2$DPR,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=min)

colnames(pf2w) = c("SID","erimweek","UPC","price")

uw = aggregate(pf2$UNITS,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=sum)

colnames(uw) = c("SID","erimweek","UPC","UNITS")

pf2w$UNITS = uw$UNITS

rp = aggregate(pf2$REG,by=list(pf2$SID,pf2$erimweek,pf2$UPC),FUN=min)

colnames(rp) = c("SID","erimweek","UPC","REG")

pf2w$REG = rp$REG

# load in prices from store data + imputations
# the prices not in the price files will be used for the two aggregate products

load(paste(outputpath,"stprices_",product,".RData",sep=""))

# fill in prices for products missing from the sas price files

# first, figure out the regular price from the data without the special price code on

dyn.load("functions.dll")

out = .Call("fillregprices",stprices)

dyn.unload("functions.dll")

# add imputed/store prices to sas imputed prices
# This is only done for market 1, since that is where we are estimating the model

stmarket = aggregate(stdata$market,by=list(stdata$storeid),FUN=max)
colnames(stmarket) = c("storeid","market")
stmarket = stmarket[order(stmarket$storeid),]

for(st in 1:length(allstids)) {
  for(br in 1:length(brandnames)) {
    for(f in 1:length(flavor)) {
      if(stmarket$market[st] == 1) {
        inds = sum(pf1w$SID == allstids[st] & pf1w$brandid == br & pf1w$flavor == f)
        if(sum(inds) == 0) {
          nobs = sum(out$DPR[st,,br,f] > 0)
          hhindx = hhpurch$storeid == allstids[st] & hhpurch$market == 1 & hhpurch$brandid == br & hhpurch$flavor == f
          if(nobs > 0 & sum(hhindx) > 0) {
            # in case of multiple brands we'll just take one upc - the one most chosen by households
            allupc = as.data.frame(table(hhpurch$upc[hhindx]),stringsAsFactors=FALSE)
            UPC = as.numeric(allupc[which.max(allupc[,2]),1])
            SID = as.integer(allstids[st])
            indxs = out$DPR[st,,br,f] > 0
            erimweek = allweek[indxs]
            price = out$DPR[st,indxs,br,f]
            nunits = aggregate(hhpurch$units[hhindx]/100,by=list(hhpurch$week[hhindx]),FUN=sum)
            colnames(nunits) = c("erimweek","units")
            wkbig = data.frame(erimweek)
            colnames(wkbig) = "erimweek"
            nunits = merge(wkbig,nunits,all.x=TRUE)
            nunits$units[is.na(nunits$units)] = 0
            UNITS = as.integer(nunits$units[order(nunits$erimweek)])
            REG = out$REG[st,indxs,br,f]
            desc = bdatabig$desc[bdatabig$upc==UPC]
            wtamt = as.integer(bdatabig$wtamt[bdatabig$upc==UPC])
            temp = data.frame(rep(UPC,nobs),rep(SID,nobs),erimweek,price,UNITS,REG,rep(desc,nobs),
                              rep(wtamt,nobs),rep(as.integer(1),nobs),rep(br,nobs),rep(f,nobs),rep(0,nobs),stringsAsFactors=FALSE)
            colnames(temp) = colnames(pf1w)
            pf1w = rbind(pf1w,temp)
          }
        }
      }
    }
  }
}

# compare sale frequencies, length, etc

brands = unique(c(pf1w$brandid,pf2w$brandid))

brands = brands[order(brands)]

scut = matrix(0,nrow=length(brands),ncol=2)
scuthh = matrix(0,nrow=length(brands),ncol=2)

j=0
for(br in brands) {
  j=j+1

  scut[j,1] = quantile(pf1w$price[pf1w$price > 0 & pf1w$brandid == br],probs=c(0.25))
  scut[j,2] = quantile(pf2w$price[pf2w$price > 0 & pf2w$brandid == br],probs=c(0.25))
  scuthh[j,1] = quantile(hhpurch$uprice[hhpurch$brandid == br & hhpurch$market == 1],probs=c(0.5))
  scuthh[j,2] = quantile(hhpurch$uprice[hhpurch$brandid == br & hhpurch$market == 2],probs=c(0.5))

}

sc = array(0.59,c(length(brands),2))

spct = array(0.05,c(length(brands),2))  # percent where we define sale

# compute promotion frequencies

pf1w$sale = FALSE
pf2w$sale = FALSE

j=0

mweek = allweek[1]
inds1 = pf1w$erimweek >= mweek
inds2 = pf2w$erimweek >= mweek

salepcts = matrix(0,nrow=length(brands),ncol=2)

for(br in brands) {
  j=j+1
  pf1w$sale[pf1w$brandid == br & pf1w$price <= pf1w$REG*(1-spct[j,1])] = TRUE
  pf2w$sale[pf2w$brandid == br & pf2w$price <= pf2w$REG*(1-spct[j,2])] = TRUE
  salepcts[j,1] = mean(pf1w$sale[pf1w$brandid == br])
  salepcts[j,2] = mean(pf2w$sale[pf2w$brandid == br])
  cat("% of time ",brandnames[br]," is on sale in market 1: ",mean(pf1w$sale[pf1w$brandid == br & inds1]),"\n",sep="")
  cat("% of time ",brandnames[br]," is on sale in market 2: ",mean(pf2w$sale[pf2w$brandid == br & inds2]),"\n",sep="")
}

# promotion depth

for(br in brands) {
  cat("avg sale, nonsale price for brand ",brandnames[br],": ",mean(pf1w$price[pf1w$brandid == br & pf1w$sale & inds1]),
      ", ",mean(pf1w$price[pf1w$brandid == br & !pf1w$sale & inds1])," for market 1.\n",sep="")
  cat("avg sale, nonsale price for brand ",brandnames[br],": ",mean(pf2w$price[pf2w$brandid == br & pf2w$sale & inds2]),
      ", ",mean(pf2w$price[pf2w$brandid == br & !pf2w$sale & inds2])," for market 2.\n",sep="")
}

avgprices = array(0,c(length(brands),3,2))

blist = list(1,2)
for(i in brands) {
  avgprices[i,1,1] = mean(pf1w$price[pf1w$brandid %in% blist[[i]] & inds1])
  avgprices[i,2,1] = mean(pf1w$price[pf1w$brandid %in% blist[[i]] & pf1w$sale & inds1])
  avgprices[i,3,1] = mean(pf1w$price[pf1w$brandid %in% blist[[i]] & !pf1w$sale & inds1])

  avgprices[i,1,2] = mean(pf2w$price[pf2w$brandid %in% blist[[i]] & inds2])
  avgprices[i,2,2] = mean(pf2w$price[pf2w$brandid %in% blist[[i]] & pf2w$sale & inds2])
  avgprices[i,3,2] = mean(pf2w$price[pf2w$brandid %in% blist[[i]] & !pf2w$sale & inds2])

}

# sale frequencies - only do these for the estimation brands

load(paste(inputpath,"/",product,"/sssoup_upcs.RData",sep=""))

nweek = length(allweek) - sum(allweek < mweek)
nmonth = as.integer(nweek/4)+1
month2week = kronecker(1:nmonth,rep(1,4))[1:nweek]

salepctmonth = array(0,c(nmonth,2))

for(i in 1:nmonth) {
  salepctmonth[i,1] = mean(pf1w$sale[pf1w$UPC %in% upckeep & pf1w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == i)]])
  salepctmonth[i,2] = mean(pf2w$sale[pf2w$UPC %in% upckeep & pf2w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == i)]])
}

# bimonthly

salepctbimonth = array(0,c(as.integer(nmonth/2)+1,2))

for(i in 1:as.integer(nmonth/2)) {
  salepctbimonth[i,1] = mean(pf1w$sale[pf1w$UPC %in% upckeep & pf1w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week %in% c(2*i-1,2*i))]])
  salepctbimonth[i,2] = mean(pf2w$sale[pf2w$UPC %in% upckeep & pf2w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week %in% c(2*i-1,2*i))]])
}

if(nmonth%%2 != 0) {
  salepctbimonth[as.integer(nmonth/2)+1,1] = mean(pf1w$sale[pf1w$UPC %in% upckeep & pf1w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == as.integer(nmonth/2)+1)]])
  salepctbimonth[as.integer(nmonth/2)+1,2] = mean(pf2w$sale[pf2w$UPC %in% upckeep & pf2w$erimweek %in% allweek[sum(allweek < mweek)+which(month2week == as.integer(nmonth/2)+1)]])
}

pf1wu = pf1w[pf1w$UPC %in% upckeep,]

pf1wu = pf1wu[order(pf1wu$SID,pf1wu$UPC,pf1wu$erimweek),]

wkindexes = match(pf1wu$erimweek,allweek)

# compute conditional probabilities of promotions

saleowncprobs = list(matrix(0,nrow=2,ncol=2),matrix(0,nrow=2,ncol=2))
salecompcprobs = list(matrix(0,nrow=2,ncol=2),matrix(0,nrow=2,ncol=2))

brids = c(1,2)

lead.pf1w = pf1w[-1,]
lag.pf1w = pf1w[-nrow(pf1w),]

# create stacked data

upclist = c(unique(pf1w$UPC[pf1w$brandid==1]), unique(pf1w$UPC[pf1w$brandid==2]))
upcbrand = c(rep(1,length(unique(pf1w$UPC[pf1w$brandid==1]))),rep(2,length(unique(pf1w$UPC[pf1w$brandid==2]))))

allstid = unique(pf1w$SID)

pf1wstack = data.frame(cbind(kronecker(allstid,rep(1,length(allweek))),rep(allweek,length(allstid))))
colnames(pf1wstack) = c("SID","erimweek")

for(i in 1:length(upclist)) {
  temp = pf1w[pf1w$UPC == upclist[i],c("SID","erimweek","sale")]
  colnames(temp)[3] = paste("sale",i,sep="")
  pf1wstack = merge(pf1wstack,temp,all.x=TRUE)
}

pf1wstack = pf1wstack[order(pf1wstack$SID,pf1wstack$erimweek),]

for(i in 1:length(brids)) {
  pf1wstack[[paste("brsale",i,sep="")]] = NA
  for(j in which(upcbrand==brids[i])) {
    inds = is.na(pf1wstack[[paste("brsale",i,sep="")]])
    pf1wstack[[paste("brsale",i,sep="")]][inds] = pf1wstack[[paste("sale",j,sep="")]][inds]
    inds = !is.na(pf1wstack[[paste("brsale",i,sep="")]]) & !is.na(pf1wstack[[paste("sale",j,sep="")]])
    pf1wstack[[paste("brsale",i,sep="")]][inds] = pf1wstack[[paste("brsale",i,sep="")]][inds] | pf1wstack[[paste("sale",j,sep="")]][inds]
  }
}

for(i in 1:length(brids)) {
  inds = is.na(pf1wstack[[paste("brsale",i,sep="")]])
  pf1wstack = pf1wstack[!inds,]
}

lead.stack = pf1wstack[-1,]
lag.stack = pf1wstack[-nrow(pf1wstack),]


for(i in 2:length(allweek)) {
  for(j in 1:length(brids)) {
    inds = lead.pf1w$erimweek == allweek[i] & lag.pf1w$erimweek == allweek[i-1] &
        lead.pf1w$SID == lag.pf1w$SID & lead.pf1w$UPC == lag.pf1w$UPC & lead.pf1w$brandid == brids[j]
    saleowncprobs[[j]][1,1] = saleowncprobs[[j]][1,1] + sum(lead.pf1w$sale[inds] & lag.pf1w$sale[inds])
    saleowncprobs[[j]][1,2] = saleowncprobs[[j]][1,2] + sum(!lead.pf1w$sale[inds] & lag.pf1w$sale[inds])
    saleowncprobs[[j]][2,1] = saleowncprobs[[j]][2,1] + sum(lead.pf1w$sale[inds] & !lag.pf1w$sale[inds])
    saleowncprobs[[j]][2,2] = saleowncprobs[[j]][2,2] + sum(!lead.pf1w$sale[inds] & !lag.pf1w$sale[inds])

    otherbrand = 3-j

    inds = lead.stack$erimweek == allweek[i] & lag.stack$erimweek == allweek[i-1] &
        lead.stack$SID == lag.stack$SID

    salecompcprobs[[j]][1,1] = salecompcprobs[[j]][1,1] + sum(lead.stack[[paste("brsale",j,sep="")]][inds] & lag.stack[[paste("brsale",otherbrand,sep="")]][inds])
    salecompcprobs[[j]][1,2] = salecompcprobs[[j]][1,2] + sum(!lead.stack[[paste("brsale",j,sep="")]][inds] & lag.stack[[paste("brsale",otherbrand,sep="")]][inds])
    salecompcprobs[[j]][2,1] = salecompcprobs[[j]][2,1] + sum(lead.stack[[paste("brsale",j,sep="")]][inds] & !lag.stack[[paste("brsale",otherbrand,sep="")]][inds])
    salecompcprobs[[j]][2,2] = salecompcprobs[[j]][2,2] + sum(!lead.stack[[paste("brsale",j,sep="")]][inds] & !lag.stack[[paste("brsale",otherbrand,sep="")]][inds])

  }
}


for(j in 1:length(brids)) {
  saleowncprobs[[j]] = saleowncprobs[[j]]/rowSums(saleowncprobs[[j]])
  salecompcprobs[[j]] = salecompcprobs[[j]]/rowSums(salecompcprobs[[j]])
}

cprobtabown = c("Sale","Nonsale")

cprobtabcomp = c("Sale","Nonsale")

sigfig = 2
for(i in 1:2) {
  for(j in 1:2) {
    for(k in 1:2) {
      cprobtabown[i] = paste(cprobtabown[i]," & ",round(saleowncprobs[[j]][i,k],sigfig),sep="")
      cprobtabcomp[i] = paste(cprobtabcomp[i]," & ",round(salecompcprobs[[j]][i,k],sigfig),sep="")
    }
  }
  cprobtabown[i] = paste(cprobtabown[i],"\\\\")
  cprobtabcomp[i] = paste(cprobtabcomp[i],"\\\\")
}


write.table(cprobtabown,paste(tablepath,"soup_saleprobs.tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(cprobtabcomp,paste(tablepath,"soup_compprobs.tex",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

# make graphs and tables of prices

# price and quantity graphs: the 2 most popular stores in each market: 7 and 30

pf1w = merge(pf1w,w2d,all.x=TRUE)
pf1w = pf1w[order(pf1w$SID,pf1w$erimweek,pf1w$UPC),]
pf1w$WEEK = pf1w$mindate
pf1w$mindate <- NULL
pf1w$maxdate <- NULL

pf2w = merge(pf2w,w2d,all.x=TRUE)
pf2w = pf2w[order(pf2w$SID,pf2w$erimweek,pf2w$UPC),]
pf2w$WEEK = pf2w$mindate
pf2w$mindate <- NULL
pf2w$maxdate <- NULL

inds = pf1w$SID == 29 & pf1w$brandid == 1 & pf1w$flavor == 3

# there is some issue with price in week 198707 - is listed as 28 cents
# but nobody in purchase data pays it (they pay 36 cents).
# same thing happens in 198832 - only person who buys pays
# 40 cents.

pp = pf1w$price[inds]*100
pp[pf1w$erimweek[inds] == 198707] = 36
pp[pf1w$erimweek[inds] == 198832] = 40

op <- par(mfrow=c(2,1))
plot(pf1w$WEEK[inds],pp,type="l",ylab="Price (cents)",xlab="Week",main="Price and Quantity of Campbell's 10.5 oz Chicken Noodle Soup",col=c("blue"))
plot(pf1w$WEEK[inds],pf1w$UNITS[inds],type="l",ylab="Quantity",xlab="Week",col=c("blue"))
par(op)
dev.copy2eps(file=paste(figpath,"pricequantitysssoupmkt1.eps",sep=""))
dev.copy2pdf(file=paste(figpath,"pricequantitysssoupmkt1.pdf",sep=""))

# save output files for step 2

save(pf1w,pf2w,file=paste(outputpath,"weeklyprices_",product,".RData",sep=""))
save(bdatabig,file=paste(outputpath,"bdatabig_",product,".RData",sep=""))
