rm(list=ls())

library(plm)

product = "sssoup"
inputpath = "C:/Users/MattOsborne/Dropbox/price index paper/step 1 output files/"                   #data from step 1
outputpath = "C:/Users/MattOsborne/Dropbox/price index paper/estimation data/"         #output data for fortran code
tablepath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"    #tables for paper
figpath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"      #figures for paper


load(paste(inputpath,"erim_",product,".RData",sep=""))
load(paste(inputpath,"weeklyprices_",product,".RData",sep=""))
load(paste(inputpath,"bdatabig_",product,".RData",sep=""))

load(file=paste(inputpath,"visits_",product,".RData",sep=""))

brands = unique(bdatabig$brandid)
brands = brands[order(brands)]

flavors = unique(bdatabig$flavor)
flavors = flavors[order(flavors)]


hhpurch$filler <- NULL

hhpurch = merge(hhpurch,bdatabig,all.x=TRUE)

# compute measures of brand loyalty
# define loyal as greater than 0.8 of all purchases

brlist = c(1,2)

hhp1 = hhpurch[,c("hhid","week","storeid","brandid")]
for(i in 1:length(brlist)) {
  hhp1[[paste("brand",i,sep="")]] = hhp1$brandid == brlist[i]
  temp = aggregate(hhp1[[paste("brand",i,sep="")]],by=list(hhp1$hhid),FUN=mean)
  colnames(temp) = c("hhid",paste("fr",i,sep=""))
  if(i==1) {
    purchfr = temp
  } else {
    purchfr[[paste("fr",i,sep="")]] = temp[[paste("fr",i,sep="")]]
  }
  cat("Fraction of households loyal to product ",i,": ",mean(purchfr[[paste("fr",i,sep="")]]>0.8),"\n")
}

stdata = merge(stdata,bdatabig,all.x=TRUE)
stdata = stdata[order(stdata$storeid,stdata$week,stdata$brandid,stdata$flavor,stdata$upc),]

stmarket = rep(0,50)
for(i in 1:50) {
  if(sum(stdata$storeid==i) > 0) {
    stmarket[i] = max(stdata$market[stdata$storeid==i])
  }
}
write.table(as.integer(stmarket),file=paste(outputpath,product,"/stmarket.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

allweek = unique(hhpurch$week)
allweek = allweek[order(allweek)]

allstids = unique(stdata$storeid[stdata$market == 1])
allstids = allstids[order(allstids)]

# aggregate data to the week-store level
hhvisit = hhvisit[hhvisit$market == 1,]
temp = aggregate(hhvisit$dollars,by=list(hhvisit$hhid,hhvisit$week,hhvisit$storeid),FUN=sum)
colnames(temp) = c("hhid","week","storeid","dollars")
hhvisit = temp

# merge purchases with visits
hhpurch = hhpurch[hhpurch$market == 1 & hhpurch$packet == 0,]
hhpurch$filler = NULL

hhall = merge(hhpurch,hhvisit,all.y=TRUE)

# merge in prices and advertising

# first construct average prices from the price files, and advertising from the store files

brpurch = aggregate(hhpurch$units,by = list(hhpurch$week,hhpurch$brandid,hhpurch$flavor),FUN=sum)
colnames(brpurch) = c("week","brandid","flavor","x")


qwts = array(0,c(length(allweek),length(brands),length(flavors)))

j=0
for(br in brands) {
  j=j+1
  k=0
  for(i in flavors) {
    k=k+1
    wks = which(allweek %in% brpurch$week[brpurch$brandid==br & brpurch$flavor==i])
    qwts[wks,j,k] = brpurch$x[brpurch$brandid==br & brpurch$flavor == i]
  }
}

avgprices = data.frame(kronecker(allstids,rep(1,length(allweek))),rep(allweek,length(allstids)))
colnames(avgprices) = c("storeid","week")
avgprices$price <- 0
avgprices$ppoz <- 0
avgprices$units <- 0
avgprices$sale <- 0
avgprices$feat <- 0
avgprices$disp <- 0
avgprices$otherad <- 0

pf1w = pf1w[pf1w$erimweek %in% allweek,]

j=0
for(br in brands) {
  j=j+1
  k=0
  for(f in flavors) {
    k=k+1
    for(st in allstids) {
      pfinds = pf1w$SID == st & pf1w$brandid == br & pf1w$flavor == f
      wks = pf1w$erimweek[pfinds]
      if(length(wks) > 0) {
        # we need to aggregate since some brand/flavor combinations cover multiple UPCs

        avp = aggregate(pf1w$price[pfinds],by=list(wks),FUN=mean)
        avppoz = aggregate(100*pf1w$price[pfinds]/pf1w$wtamt[pfinds],by=list(wks),FUN=mean)
        msale = aggregate(pf1w$sale[pfinds],by=list(wks),FUN=max)

        inds = avgprices$storeid == st & avgprices$week %in% wks
        if(sum(inds) != length(avp$x)) stop("length bug")

        avgprices$price[inds] = avgprices$price[inds] + qwts[allweek %in% wks,j,k]*avp$x
        avgprices$ppoz[inds] = avgprices$ppoz[inds] + qwts[allweek %in% wks,j,k]*avppoz$x
        avgprices$units[inds] = avgprices$units[inds] + qwts[allweek %in% wks,j,k]
        avgprices$sale[inds] = pmax(avgprices$sale[inds], msale$x)

        hhinds = hhpurch$brandid == br & hhpurch$flavor == f & hhpurch$storeid == st
        if(sum(hhinds) > 0) {
          disp1 = hhpurch$eadc[hhinds] == "Y" | hhpurch$iadc[hhinds] == "Y" | hhpurch$fadc[hhinds] == "Y" | hhpurch$odc[hhinds] == "Y"
          disp2 = aggregate(disp1,by=list(hhpurch$week[hhinds]),FUN=max)
          inds2 = avgprices$storeid == st & avgprices$week %in% disp2[,1]
          avgprices$disp[inds2] = pmax(avgprices$disp[inds2],disp2$x)
          feat1 = hhpurch$adtype[hhinds] == "L" | hhpurch$adtype[hhinds] == "M"
          feat2 = aggregate(feat1,by=list(hhpurch$week[hhinds]),FUN=max)
          inds2 = avgprices$storeid == st & avgprices$week %in% feat2[,1]
          avgprices$feat[inds2] = pmax(avgprices$feat[inds2],feat2$x)
          other1 = hhpurch$popcode[hhinds] == "Y" #hhpurch$inadcc[hhinds] == "Y" #| hhpurch$popcode[hhinds] == "Y"
          other2 = aggregate(other1,by=list(hhpurch$week[hhinds]),FUN=max)
          inds2 = avgprices$storeid == st & avgprices$week %in% other2[,1]
          avgprices$otherad[inds2] = pmax(avgprices$otherad[inds2],other2$x)
        }
        stinds = stdata$brandid == br & stdata$flavor == f & stdata$storeid == st
        if(sum(stinds) > 0) {
          disp1 = stdata$eadc[stinds] == "Y" | stdata$iadc[stinds] == "Y" | stdata$fadc[stinds] == "Y" | stdata$odc[stinds] == "Y"
          disp2 = aggregate(disp1,by=list(stdata$week[stinds]),FUN=max)
          inds2 = avgprices$storeid == st & avgprices$week %in% disp2[,1]
          avgprices$disp[inds2] = pmax(avgprices$disp[inds2],disp2$x)
          feat1 = stdata$adcode[stinds] == "L" | stdata$adcode[stinds] == "M"
          feat2 = aggregate(feat1,by=list(stdata$week[stinds]),FUN=max)
          inds2 = avgprices$storeid == st & avgprices$week %in% feat2[,1]
          avgprices$feat[inds2] = pmax(avgprices$feat[inds2],feat2$x)
          other1 = stdata$popcode[stinds] == "Y" #stdata$inadcc[stinds] == "Y" #| stdata$popcode[stinds] == "Y"
          other2 = aggregate(other1,by=list(stdata$week[stinds]),FUN=max)
          inds2 = avgprices$storeid == st & avgprices$week %in% other2[,1]
          avgprices$otherad[inds2] = pmax(avgprices$otherad[inds2],other2$x)
        }

      }
    }
  }
}

avgprices$price[avgprices$units > 0] = avgprices$price[avgprices$units > 0]/avgprices$units[avgprices$units > 0]
avgprices$ppoz[avgprices$units > 0] = 100*avgprices$ppoz[avgprices$units > 0]/avgprices$units[avgprices$units > 0]
avgprices$units = NULL

hhall$pprice = hhall$price
hhall$price = NULL

hhall = merge(hhall,avgprices,all.x=TRUE)

hhall$units[is.na(hhall$units)] = 0
hhall$wtamt[is.na(hhall$wtamt)] = 0

# compute household consumption rates, and then inventories

# first collapse data to the week level
hhall1 = hhall[hhall$price > 0,]
dollars1 = aggregate(hhall1$dollars,by=list(hhall1$hhid,hhall1$week),FUN=max)
colnames(dollars1) = c("hhid","week","maxdollars")
hhall1 = merge(hhall1,dollars1)
rm(dollars1)

totunits = aggregate((hhall1$units/100)*(hhall1$wtamt/1000),by=list(hhall1$hhid,hhall1$week),FUN=sum)   #THIS IS IMPORTANT UNITS MUST BE MULTIPLIED BY WEIGHT
colnames(totunits) = c("hhid","week","totunits")
hhall1 = merge(hhall1,totunits)

hhall1 = hhall1[(hhall1$totunits > 0 & hhall1$units > 0) | (hhall1$totunits == 0 & hhall1$dollars == hhall1$maxdollars),]

price1 = aggregate(hhall1$price,by=list(hhall1$hhid,hhall1$week),FUN=mean)
colnames(price1) = c("hhid","week","price")

ppoz1 = aggregate(hhall1$ppoz,by=list(hhall1$hhid,hhall1$week),FUN=mean)
colnames(ppoz1) = c("hhid","week","ppoz")

feat1 = aggregate(hhall1$feat,by=list(hhall1$hhid,hhall1$week),FUN=max)
colnames(feat1) = c("hhid","week","feat")

disp1 = aggregate(hhall1$disp,by=list(hhall1$hhid,hhall1$week),FUN=max)
colnames(disp1) = c("hhid","week","disp")

other1 = aggregate(hhall1$otherad,by=list(hhall1$hhid,hhall1$week),FUN=max)
colnames(other1) = c("hhid","week","otherad")

sale1 = aggregate(hhall1$sale,by=list(hhall1$hhid,hhall1$week),FUN=max)
colnames(sale1) = c("hhid","week","sale")

fldum = vector("list",length=length(flavors))
j=0
for(f in flavors) {
  j=j+1
  fldum[[j]] = aggregate(hhall1$flavor==f,by=list(hhall1$hhid,hhall1$week),FUN=max)
  colnames(fldum[[j]]) = c("hhid","week","x")
}

brdum = vector("list",length=length(brands))
hhall1$brandid[is.na(hhall1$brandid)] = -1

j=0
for(br in brands) {
  j=j+1
  brdum[[j]] = aggregate(hhall1$brandid==br,by=list(hhall1$hhid,hhall1$week),FUN=max)
  colnames(brdum[[j]]) = c("hhid","week","x")
}

stdum = vector("list",length=length(allstids))

j=0
for(st in allstids) {
  j=j+1
  stdum[[j]] = aggregate(hhall1$storeid==st,by=list(hhall1$hhid,hhall1$week),FUN=max)
  colnames(stdum[[j]]) = c("hhid","week","x")
1}

hhallw = price1
hhallw$ppoz = ppoz1$ppoz
hhallw$feat = feat1$feat
hhallw$disp = disp1$disp
hhallw$otherad = other1$otherad
hhallw$sale = sale1$sale
hhallw$units = totunits$totunits

j=0
for(br in brands) {
  j=j+1
  hhallw[[paste("br",br,sep="")]] = brdum[[j]]$x
}

j=0
for(f in flavors) {
  j=j+1
  hhallw[[paste("fl",f,sep="")]] = fldum[[j]]$x
}

j=0
for(st in allstids) {
  j=j+1
  hhallw[[paste("st",st,sep="")]] = stdum[[j]]$x
}

hhallw$units1 = hhallw$units/100

fmla1 = paste("units~ppoz+sale+sale*ppoz+feat+disp+",paste(paste("br",brands[c(-1)],sep=""),collapse="+"),"+",
    paste(paste("fl",flavors[c(-1)],sep=""),collapse="+"),"+",paste(paste("st",allstids[c(-1)],sep=""),collapse="+"),sep="")
print(summary(lm(as.formula(fmla1),data=hhallw)))

# compute household level consumption rates

wkindex = rep(0,nrow(hhallw))

for(i in 1:length(allweek)) {
  wkindex[hhallw$week == allweek[i]] = i
}


minweek = aggregate(wkindex,by=list(hhallw$hhid),FUN=min)
maxweek = aggregate(wkindex,by=list(hhallw$hhid),FUN=max)
hhunits = aggregate(hhallw$units,by=list(hhallw$hhid),FUN=sum)

hhcrate = hhunits$x/(maxweek$x-minweek$x+1)

hhallw = hhallw[order(hhallw$hhid,hhallw$week),]

dyn.load("functions.dll")

inv = .Call("computeinv",hhallw,hhcrate)

dyn.unload("functions.dll")

hhallw$inv = inv
hhallw$inv1 = inv/100

fmla2 = paste(fmla1,"inv",sep="+")

res = plm(as.formula(fmla2),data=hhallw,index=c("hhid","week"))

print(summary(res))

coefsoup = summary(res)$coefficients

save(coefsoup,file=paste(outputpath,product,"/coef",product,".RData",sep=""))

# compute indexes plus imputed inventories

# first find stores that have complete price series

stweeks = array(0,c(length(allstids),length(brands),length(flavors)))

startweek = 88
k=0
for(st in allstids) {
  k=k+1
  i=0
  for(br in brands) {
    i=i+1
    for(j1 in 1:length(flavors)) {
      j = which(j1 == flavors)
      stweeks[k,i,j] = sum(pf1w$erimweek >= allweek[startweek] & pf1w$SID == st & pf1w$brandid == br & pf1w$flavor == j)
    }
  }
}

stcomplete = rep(TRUE, length(allstids))
tdelta = 0

binclude = c(1)
flinclude = c(1,2,3)

k=0
for(st in allstids) {
  k=k+1
  i=0
  for(br in binclude) {
    i=which(br==brands)
    for(j1 in flinclude) {
      j=which(j1==flavors)
      stcomplete[k] = stcomplete[k] & stweeks[k,i,j] >= length(allweek)-startweek+1-tdelta
    }
  }
}

# check for evidence of nonlinear pricing

# check the daily file to make sure nothing is missing

load(paste(inputpath,"dailyprices1_",product,".RData",sep=""))

pf1 = pf1[pf1$UPC %in% unique(pf1w$UPC),]

load(paste(inputpath,"d2w.RData",sep=""))

pf1$erimweek = 0
for(i in 1:nrow(d2w)) {
  pf1$erimweek[pf1$DATE >= d2w$mindate[i] & pf1$DATE <= d2w$maxdate[i]] = d2w$erimweek[i]
}


pf1$dayofweek = weekdays(pf1$DATE)
pf1$erimday = 1
j=1
for(i in c("Monday","Tuesday","Wednesday","Thursday","Friday","Saturday")) {
  j=j+1
  pf1$erimday[pf1$dayofweek == i] = j
}

pcheck = pf1
pcheck = pcheck[,c("erimweek","erimday","UPC","SID","DPR","REG")]
colnames(pcheck) = c("week","day","upc","storeid","stprice","REG")

pmerge = merge(hhpurch,pcheck)
pmerge$ppunit = pmerge$price/pmerge$units
pmerge$ppunit[is.na(pmerge$ppunit)] = pmerge$stprice[is.na(pmerge$ppunit)]  #only one observation

ii = pmerge$units > 100

sum( abs(pmerge$ppunit[ii] - pmerge$stprice[ii]) > 0.01 )/nrow(pmerge[ii,])  # this matches up well

badobs = abs(pmerge$ppunit - pmerge$stprice) > 0.01 & ii

upccheck = unique(pf1$UPC)

hhpurch1 = aggregate(hhpurch$price,by=list(hhpurch$upc,hhpurch$storeid,hhpurch$week,hhpurch$day,hhpurch$units),FUN=mean)

colnames(hhpurch1) = c("upc","storeid","week","day","units","avgprice")

hhpurch1 = hhpurch1[hhpurch1$upc %in% upccheck,]

punit = vector("list",length=6)

for(i in 1:6) {

  punit[[i]] = hhpurch1[hhpurch1$units==i*100,]
  colnames(punit[[i]]) = c("upc","storeid","week","day","units",paste("p",i,sep=""))
  punit[[i]]$units <- NULL

}

for(i in 1:6) {

  hhpurch1 = merge(hhpurch1,punit[[i]],all.x=TRUE)
  hhpurch1[[paste("p",i,sep="")]][is.na(hhpurch1[[paste("p",i,sep="")]])] = 0

}

mmat = matrix(0,nrow=6,ncol=6)
difmat = matrix(0,nrow=6,ncol=6)
pdifmat = array(0,c(6,6,2))
remainders = matrix(0,nrow=6,ncol=2)

for(i in 1:6) {
  remainders[i,1] = sum(hhpurch1[[paste("p",i,sep="")]]%%i > 0)
  remainders[i,2] = sum(hhpurch1[[paste("p",i,sep="")]] > 0)
  for(j in 1:6) {
    ii = (hhpurch1[[paste("p",i,sep="")]] > 0) & (hhpurch1[[paste("p",j,sep="")]] > 0)
    mmat[i,j] = sum( ii )
    difmat[i,j] = sum( hhpurch1[[paste("p",i,sep="")]][ii]/i != hhpurch1[[paste("p",j,sep="")]][ii]/j )
    ii2 = hhpurch1[[paste("p",i,sep="")]]/i != hhpurch1[[paste("p",j,sep="")]]/j & ii
    pdifmat[i,j,1] = mean(hhpurch1[[paste("p",i,sep="")]][ii2]/i)
    pdifmat[i,j,2] = mean(hhpurch1[[paste("p",j,sep="")]][ii2]/j)
  }
}

print(table(stdata$pmultiple[stdata$market==1]))
print(sum(stdata$pmultiple[stdata$market==1] > 1)/sum(stdata$market==1))

print(remainders)
print(sum(remainders[2:6,1])/sum(remainders[2:6,]))
print(pdifmat)

pdiftab = rep(paste(1:5," Unit(s)",sep=""))

for(i in 1:5) {
  for(j in 1:6) {
    if(difmat[i,j] == 0 | i > j) {
      pdiftab[i] = paste(pdiftab[i]," & - ",sep="")
    } else {
      pdiftab[i] = paste(pdiftab[i]," & ",round(100*difmat[i,j]/mmat[i,j],1)," (",round(abs(pdifmat[i,j,1]-pdifmat[i,j,2]),1),") ",sep="")
    }
  }
  pdiftab[i] = paste(pdiftab[i]," \\\\",sep="")
}

write.table(pdiftab,file=paste(tablepath,"ditftab_soup.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

minweek = allweek[88]

qcutoff = 5

maxu = 20

# these are in order of market share.

allupc = data.frame(table(hhpurch$upc),stringsAsFactors=FALSE)
allupc[,1] = as.numeric(as.character(allupc[,1]))
colnames(allupc) = c("upc","Freq")
allupc = allupc[order(allupc$Freq,decreasing=TRUE),]

upckeep = allupc$upc[1:10]

commoncols = colnames(hhall1)[colnames(hhall1) %in% colnames(hhpurch)]
missingcols = colnames(hhall1)[!(colnames(hhall1) %in% colnames(hhpurch))]

hhpurch1 = hhpurch[hhpurch$week < minweek,]
hhpurch1 = hhpurch1[hhpurch1$weight >= 10500 & hhpurch1$weight <= 11500,]

agupcs = vector("list",length=2)

hhall1.old = hhall1
agupcs[[1]] = unique(hhpurch1$upc[ !(hhpurch1$upc %in% upckeep) & hhpurch1$brandid == 1 & hhpurch1$weight >= 10500 & hhpurch1$weight <= 11500])
hhpurch1$upc[ !(hhpurch1$upc %in% upckeep) & hhpurch1$brandid == 1 & hhpurch1$weight >= 10500 & hhpurch1$weight <= 11500] = 1
agupcs[[2]] = unique(hhpurch1$upc[ !(hhpurch1$upc %in% upckeep) & hhpurch1$brandid == 2 & hhpurch1$weight >= 10500 & hhpurch1$weight <= 11500])
hhpurch1$upc[ !(hhpurch1$upc %in% upckeep) & hhpurch1$brandid == 2 & hhpurch1$weight >= 10500 & hhpurch1$weight <= 11500] = 2
hhall1$upc[ !(hhall1$upc %in% upckeep) & hhall1$brandid == 1 & hhall1$weight >= 10500 & hhall1$weight <= 11500] = 1
hhall1$upc[ !(hhall1$upc %in% upckeep) & hhall1$brandid == 2 & hhall1$weight >= 10500 & hhall1$weight <= 11500 ] = 2
upckeep = c(upckeep,1,2)

tunits1 = aggregate(hhpurch1$units,by=list(hhpurch1$hhid,hhpurch1$week),FUN=sum)
colnames(tunits1) = c("hhid","week","tunits")

# collapsing the data - it doesn't matter how we do this, since we're not dealing
# with brand specific inventory.  The store might differ if there are
# 2 different store purchases in a week, but this is rare

hhpurch1$obsnum = 1:nrow(hhpurch1)
maxobs = aggregate(hhpurch1$obsnum,by=list(hhpurch1$hhid,hhpurch1$week),FUN=max)
colnames(maxobs) = c("hhid","week","maxobs")

hhpurch1 = merge(hhpurch1,maxobs,all.x=TRUE)

hhpurch1 = merge(hhpurch1,tunits1,all.x=TRUE)

hhpurch1 = hhpurch1[hhpurch1$obsnum == hhpurch1$maxobs,]
hhpurch1$obsnum <- NULL
hhpurch1$maxobs <- NULL

hhpurch1$units = hhpurch1$tunits
hhpurch1$tunits <- NULL

hhpurch1 = hhpurch1[,commoncols]
for(i in missingcols) {
  hhpurch1[[i]] = 0
}


hhall2 = rbind(hhpurch1[hhpurch1$week < minweek,],hhall1[,c(commoncols,missingcols)])

hhall2$upc[is.na(hhall2$upc)] = 0

badobs1 = !(  hhall2$upc == 0 | (hhall2$upc %in% upckeep & hhall2$weight >= 10500 & hhall2$weight <= 11500) |
                ( !(hhall2$upc %in% upckeep) & (hhall2$weight < 10500 | hhall2$weight > 11500)  ) )

hhall2$totb = 0

for(i in 1:length(upckeep)) {
  hhall2[[paste("b",i,sep="")]] = hhall2$upc == upckeep[i]
  bweek = aggregate(hhall2[[paste("b",i,sep="")]],by=list(hhall2$hhid,hhall2$week),FUN=max)
  colnames(bweek) = c("hhid","week",paste("mb",i,sep=""))
  hhall2 = merge(hhall2,bweek,all.x=TRUE)
  hhall2$totb = hhall2$totb + hhall2[[paste("mb",i,sep="")]]
  hhall2[[paste("ub",i,sep="")]] = 0
  hhall2[[paste("ub",i,sep="")]][hhall2$upc == upckeep[i]] = hhall2$units[hhall2$upc == upckeep[i]]/100
  ubweek = aggregate(hhall2[[paste("ub",i,sep="")]],by=list(hhall2$hhid,hhall2$week),FUN=sum)
  colnames(ubweek) = c("hhid","week",paste("sumub",i,sep=""))
  hhall2 = merge(hhall2,ubweek,all.x=TRUE)
}

# figure out what UPCs get bought together

upcmultiple = matrix(0,nrow=length(upckeep),ncol=length(upckeep))

for(i in 1:length(upckeep)) {
  for(j in i:length(upckeep)) {
    upcmultiple[i,j] = sum(hhall2[[paste("mb",i,sep="")]] > 0 & hhall2[[paste("mb",j,sep="")]] > 0)
  }
}

# design the choice set

nmultiple = 4  # number of combos we let you buy, should also correspond to inclusive values

cratios = rbind(c(1,0,0),c(1,1,0),c(1,2,0),c(1,3,0),c(1,1,1))

#index purchases by cratio

hhall2$cratio = -1
hhall2$cratio[hhall2$totb==0] = 0
hhall2$cratio[hhall2$totb==1] = 1
inds = hhall2$totb==2
for(i in 1:(length(upckeep)-1)) {
  for(j in (i+1):length(upckeep)) {
    inds1 = inds & hhall2[[paste("sumub",i,sep="")]] == hhall2[[paste("sumub",j,sep="")]] & hhall2[[paste("sumub",i,sep="")]] > 0
    hhall2$cratio[inds1] = 2
  }
}

for(i in 1:(length(upckeep)-1)) {
  for(j in (i+1):length(upckeep)) {
    inds1 = inds & ((hhall2[[paste("sumub",i,sep="")]]/hhall2[[paste("sumub",j,sep="")]] == 2) |
    (hhall2[[paste("sumub",i,sep="")]]/hhall2[[paste("sumub",j,sep="")]] == 0.5))  &
    (hhall2[[paste("sumub",i,sep="")]] > 0 & hhall2[[paste("sumub",j,sep="")]] > 0)
    hhall2$cratio[inds1] = 3
  }
}

for(i in 1:(length(upckeep)-1)) {
  for(j in (i+1):length(upckeep)) {
    inds1 = inds & ((hhall2[[paste("sumub",i,sep="")]]/hhall2[[paste("sumub",j,sep="")]] == 3) |
    (hhall2[[paste("sumub",i,sep="")]]/hhall2[[paste("sumub",j,sep="")]] == 1/3))  &
    (hhall2[[paste("sumub",i,sep="")]] > 0 & hhall2[[paste("sumub",j,sep="")]] > 0)
    hhall2$cratio[inds1] = 4
  }
}

tunits2 = aggregate(hhall2$units,by=list(hhall2$hhid,hhall2$week),FUN=sum)
colnames(tunits2) = c("hhid","week","tunits2")

hhall2 = merge(hhall2,tunits2,all.x=TRUE)

badobs2 = hhall2$totb > 1

badobs2.5 = !(hhall2$cratio >= 0 & hhall2$tunits2 <= 100*maxu & hhall2$totb < 3)

badobs = badobs1 | badobs2

badobsb = badobs1 | badobs2.5

badhh1 = aggregate(badobs1[hhall2$week >= minweek],by=list(hhall2$hhid[hhall2$week >= minweek]),FUN=max)

badhh2 = aggregate(badobs2[hhall2$week >= minweek],by=list(hhall2$hhid[hhall2$week >= minweek]),FUN=max)

badhh = aggregate(badobs[hhall2$week >= minweek],by=list(hhall2$hhid[hhall2$week >= minweek]),FUN=max)
colnames(badhh) = c("hhid","badhh")

badhhb = aggregate(badobsb[hhall2$week >= minweek],by=list(hhall2$hhid[hhall2$week >= minweek]),FUN=max)
colnames(badhhb) = c("hhid","badhh")

hhall2 = merge(hhall2,badhhb,all.x=TRUE)

npurchweek = aggregate( hhall2$totb[hhall2$week >= minweek] > 0, by=list(hhall2$hhid[hhall2$week >= minweek],hhall2$week[hhall2$week >= minweek]), FUN=max )

colnames(npurchweek) = c("hhid","week","npweek")

hhall2 = merge(hhall2,npurchweek,all.x=TRUE)

npurchhh = aggregate( hhall2$npweek[hhall2$week >= minweek] > 0,  by=list(hhall2$hhid[hhall2$week >= minweek]), FUN=sum)

colnames(npurchhh) = c("hhid","nphh")

hhall2 = merge(hhall2,npurchhh,all.x=TRUE)

# now start creating a final sample

finalhh = hhall2$badhh == 0 & hhall2$nphh >= qcutoff

hhall3 = hhall2[finalhh,]

# drop out purchases of really big or really small sizes - assumption is they are not substitutes

hhall3$weight[is.na(hhall3$weight)] = 0

hhall3 = hhall3[!((!(hhall3$upc %in% upckeep) & (hhall3$weight < 10500 | hhall3$weight > 11500) & hhall3$weight > 0)),]

# we still want to limit to hhs where purch occasions are from 1 store in 1 week and less than 10 units, and you make 25 or more store visits

nunitswk = aggregate(hhall3$units/100,by=list(hhall3$hhid,hhall3$week), FUN=sum)
colnames(nunitswk) = c("hhid","week","unitswk")

hhall3 = merge(hhall3,nunitswk,all.x=TRUE)

nmaxu = aggregate(hhall3$unitswk[hhall3$week >= minweek] > maxu, by = list(hhall3$hhid[hhall3$week >= minweek]), FUN = max)
colnames(nmaxu) = c("hhid","nmaxu")

hhall3 = merge(hhall3,nmaxu,all.x=TRUE)

hhall3 = hhall3[hhall3$nmaxu == 0,]

nobs2 = aggregate(hhall3$hhid,by=list(hhall3$hhid,hhall3$week,hhall3$storeid),FUN=length)
colnames(nobs2) = c("hhid","week","storeid","nobswkst")

nobs2.5 = aggregate(nobs2$nobswkst,by = list(nobs2$hhid,nobs2$week), FUN=length)
colnames(nobs2.5) = c("hhid","week","nobswk")

nobs3 = aggregate(nobs2.5$nobswk,by=list(nobs2.5$hhid),FUN=max)
colnames(nobs3) = c("hhid","maxobs")

hhall3 = merge(hhall3,nobs3,all.x=TRUE)

hhall3 = hhall3[hhall3$maxobs == 1,]

nobs4 = aggregate(hhall3$hhid[hhall3$week >= minweek],by=list(hhall3$hhid[hhall3$week >= minweek]),FUN=length)
colnames(nobs4) = c("hhid","hhobs")

hhall3 = merge(hhall3,nobs4)
hhall3 = hhall3[hhall3$hhobs >= 25,]

# this still needs to be collapsed since there are some times when a store is visited more than once

# also, we have to account for the fact you can buy multiple UPCs

# first index the upcs, and index up to 3 brands purchased
# in order of the index

hhall3$brindex = length(upckeep)+1
for(i in 1:length(upckeep)) {
  hhall3$brindex[hhall3$upc == upckeep[i]] = i
}

bpurch1 = aggregate(hhall3$brindex,by=list(hhall3$hhid,hhall3$week),FUN=min)
colnames(bpurch1) = c("hhid","week","bpurch1")

hhall3 = merge(hhall3,bpurch1)
hhall3$brindex1 = hhall3$brindex
hhall3$brindex1[hhall3$brindex1==hhall3$bpurch1] = length(upckeep)+1

bpurch2 = aggregate(hhall3$brindex1,by=list(hhall3$hhid,hhall3$week),FUN=min)
colnames(bpurch2) = c("hhid","week","bpurch2")

hhall3 = merge(hhall3,bpurch2)
hhall3$brindex2 = hhall3$brindex1
hhall3$brindex2[hhall3$brindex2==hhall3$bpurch2] = length(upckeep)+1

bpurch3 = aggregate(hhall3$brindex2,by=list(hhall3$hhid,hhall3$week),FUN=min)
colnames(bpurch3) = c("hhid","week","bpurch3")

hhall3 = merge(hhall3,bpurch3)

hhall3max = aggregate(hhall3[,c("storeid","upc","market","day","bpurch1","bpurch2","bpurch3")],by=list(hhall3$hhid,hhall3$week),FUN=max)
colnames(hhall3max) = c("hhid","week","storeid","upc","market","day","bpurch1","bpurch2","bpurch3")

hhall3sum1 = aggregate(hhall3$units*as.numeric(hhall3$brindex==hhall3$bpurch1),by=list(hhall3$hhid,hhall3$week),FUN=sum)
colnames(hhall3sum1) = c("hhid","week","units1")

hhall3sum2 = aggregate(hhall3$units*as.numeric(hhall3$brindex==hhall3$bpurch2),by=list(hhall3$hhid,hhall3$week),FUN=sum)
colnames(hhall3sum2) = c("hhid","week","units2")

hhall3sum3 = aggregate(hhall3$units*as.numeric(hhall3$brindex==hhall3$bpurch3),by=list(hhall3$hhid,hhall3$week),FUN=sum)
colnames(hhall3sum3) = c("hhid","week","units3")

hhall3max = hhall3max[order(hhall3max$hhid,hhall3max$week),]
hhall3sum1 = hhall3sum1[order(hhall3sum1$hhid,hhall3sum1$week),]
hhall3sum2 = hhall3sum2[order(hhall3sum2$hhid,hhall3sum2$week),]
hhall3sum3 = hhall3sum3[order(hhall3sum3$hhid,hhall3sum3$week),]

hhall3 = cbind(hhall3max,hhall3sum1$units1,hhall3sum2$units2,hhall3sum3$units3)
colnames(hhall3)[length(colnames(hhall3))-3+1:3] = paste("units",1:3,sep="")

hhall3$bpurch1[hhall3$bpurch1 == length(upckeep)+1] = 0
hhall3$bpurch2[hhall3$bpurch2 == length(upckeep)+1] = 0
hhall3$bpurch3[hhall3$bpurch3 == length(upckeep)+1] = 0

# now merge in prices, feature, display, etc

# when purchases are observed we have daily data.  When not we have weekly data.  First, we will take the price
# file's advertising variables and make them weekly.

# note this step is a bit different than the tuna data.  we don't have many UPCs in the daily files and lots of prices were imputed
# analyze_prices didn't impute advertising so when it is missing we will impute it

priceold = pf1w

pf1w = pf1w[order(pf1w$UPC,pf1w$SID,pf1w$erimweek),]

MAJOR = aggregate(pf1$MAJOR,by = list(pf1$UPC,pf1$SID,pf1$erimweek), FUN=max)
colnames(MAJOR) = c("UPC","SID","erimweek","MAJOR")
MAJOR = MAJOR[MAJOR$erimweek > 0 & MAJOR$erimweek < 198724,]

pf1w = merge(pf1w,MAJOR,all.x=TRUE)

LINE = aggregate(pf1$LINE,by = list(pf1$UPC,pf1$SID,pf1$erimweek), FUN=max)
colnames(LINE) = c("UPC","SID","erimweek","LINE")
LINE = LINE[LINE$erimweek > 0 & LINE$erimweek < 198724,]

pf1w = merge(pf1w,LINE,all.x=TRUE)

DISPLAY = aggregate(pf1$DISPLAY,by = list(pf1$UPC,pf1$SID,pf1$erimweek), FUN=max)
colnames(DISPLAY) = c("UPC","SID","erimweek","DISPLAY")
DISPLAY = DISPLAY[DISPLAY$erimweek > 0 & DISPLAY$erimweek < 198724,]

pf1w = merge(pf1w,DISPLAY,all.x=TRUE)

pf1w = pf1w[order(pf1w$UPC,pf1w$SID,pf1w$erimweek),]

# bring in ad data from households

hhdispad = as.numeric(hhpurch$eadc == "Y" | hhpurch$fadc == "Y" | hhpurch$iadc == "Y" | hhpurch$odc == "Y")

pf1upc = unique(pf1w$UPC)

for(st in allstids) {
  for(upc in pf1upc) {
    inds = pf1w$SID == st & pf1w$UPC == upc
    if(sum(!is.na(pf1w$MAJOR[inds])) == 0) {
      hhindx = hhpurch$storeid == st & hhpurch$upc == upc
      if(sum(hhindx) > 0) {
        hhmajor = aggregate(as.numeric(hhpurch$adtype[hhindx] == "M"),by=list(hhpurch$week[hhindx]),FUN=max)
        colnames(hhmajor) = c("erimweek","MAJOR")
        pfweek = data.frame(pf1w$erimweek[inds],stringsAsFactors=FALSE)
        colnames(pfweek) = "erimweek"
        pfweek = merge(pfweek,hhmajor,all.x=TRUE)
        hhline = aggregate(as.numeric(hhpurch$adtype[hhindx] == "L"),by=list(hhpurch$week[hhindx]),FUN=max)
        colnames(hhline) = c("erimweek","LINE")
        pfweek = merge(pfweek,hhline,all.x=TRUE)
        hhdisplay = aggregate(hhdispad[hhindx],by=list(hhpurch$week[hhindx]),FUN=max)
        colnames(hhdisplay) = c("erimweek","DISPLAY")
        pfweek = merge(pfweek,hhdisplay,all.x=TRUE)
        pfweek = pfweek[order(pfweek$erimweek),]
        pfweek$MAJOR[is.na(pfweek$MAJOR)] = 0
        pfweek$LINE[is.na(pfweek$LINE)] = 0
        pfweek$DISPLAY[is.na(pfweek$DISPLAY)] = 0
        pf1w$MAJOR[inds] = pfweek$MAJOR
        pf1w$LINE[inds] = pfweek$LINE
        pf1w$DISPLAY[inds] = pfweek$DISPLAY
      } else {
        pf1w$MAJOR[inds] = 0
        pf1w$LINE[inds] = 0
        pf1w$DISPLAY[inds] = 0
      }
    }
  }
}

pf1w$MAJOR[is.na(pf1w$MAJOR)] = 0
pf1w$LINE[is.na(pf1w$LINE)] = 0
pf1w$DISPLAY[is.na(pf1w$DISPLAY)] = 0

# construct aggregrate price measures for smaller UPCs

# using purchase data
for(i in 1:2) {
    hhindx = !(hhpurch$upc %in% upckeep) & hhpurch$brandid == i
    pf1temp = aggregate(cbind(hhpurch$price[hhindx]/hhpurch$units[hhindx],as.numeric(hhpurch$adtype[hhindx] == "M"),as.numeric(hhpurch$adtype[hhindx] == "L"),
                              as.numeric(hhdispad[hhindx])),by=list(hhpurch$storeid[hhindx],hhpurch$week[hhindx]),FUN=mean)
    colnames(pf1temp) = c("SID","erimweek","price","MAJOR","LINE","DISPLAY")
    pf1temp$REG = pf1temp$price
    pf1temp = pf1temp[order(pf1temp$SID,pf1temp$erimweek),]

    pf1units = aggregate(hhpurch$units[hhindx]/100,by=list(hhpurch$storeid[hhindx],hhpurch$week[hhindx]),FUN=sum)
    colnames(pf1units) = c("SID","erimweek","UNITS")
    pf1units = pf1units[order(pf1units$SID,pf1units$erimweek),]

    n = nrow(pf1temp)
    pf1final = data.frame(pf1temp$erimweek,rep(i,n),pf1temp[,c("SID","price")],pf1units$UNITS,pf1temp$REG,
                          rep(paste("Composite brand",i),n),as.integer(rep(11000,n)),as.integer(rep(1,n)),rep(i,n),rep(0,n),rep(0,n),
                          rep(FALSE,n),rep(as.Date("12/29/1985",format="%m/%d/%Y"),n),pf1temp[,c("MAJOR","LINE","DISPLAY")],stringsAsFactors=FALSE)
    colnames(pf1final) = colnames(pf1w)
    pf1w = rbind(pf1w,pf1final)

}
# a few UPCs are missing - add these in.
for(i in 1:(length(upckeep)-2)) {
    for(j in unique(pf1w$SID)) {
        if(sum(pf1w$UPC == upckeep[i] & pf1w$SID == j) == 0 & sum(hhpurch$upc == upckeep[i] & hhpurch$storeid == j) > 0) {
            hhindx = hhpurch$upc == upckeep[i] & hhpurch$storeid == j
            pf1temp = aggregate(cbind(hhpurch$price[hhindx]/hhpurch$units[hhindx],as.numeric(hhpurch$adtype[hhindx] == "M"),as.numeric(hhpurch$adtype[hhindx] == "L"),
                                      as.numeric(hhdispad[hhindx])),by=list(hhpurch$storeid[hhindx],hhpurch$week[hhindx]),FUN=mean)
            colnames(pf1temp) = c("SID","erimweek","price","MAJOR","LINE","DISPLAY")
            pf1temp$REG = pf1temp$price
            pf1temp = pf1temp[order(pf1temp$SID,pf1temp$erimweek),]

            pf1units = aggregate(hhpurch$units[hhindx]/100,by=list(hhpurch$storeid[hhindx],hhpurch$week[hhindx]),FUN=sum)
            colnames(pf1units) = c("SID","erimweek","UNITS")
            pf1units = pf1units[order(pf1units$SID,pf1units$erimweek),]

            n = nrow(pf1temp)
            pf1final = data.frame(pf1temp$erimweek,rep(upckeep[i],n),pf1temp[,c("SID","price")],pf1units$UNITS,pf1temp$REG,
                                  rep(hhpurch$desc[hhindx][1],n),as.integer(rep(hhpurch$wtamt[hhindx][1],n)),as.integer(rep(1,n)),
                                  rep(hhpurch$brandid[hhindx][1],n),rep(0,n),rep(0,n),
                                  rep(FALSE,n),rep(as.Date("12/29/1985",format="%m/%d/%Y"),n),pf1temp[,c("MAJOR","LINE","DISPLAY")],stringsAsFactors=FALSE)
            colnames(pf1final) = colnames(pf1w)
            pf1w = rbind(pf1w,pf1final)
        }
    }
}

hhall3$totunits = hhall3$units1+hhall3$units2+hhall3$units3


hhall3.1 = hhall3[hhall3$totunits > 0,]

for(i in 1:length(upckeep)) {
  pf1temp = pf1w[pf1w$UPC == upckeep[i],c("erimweek","SID","price","MAJOR","LINE","DISPLAY")]
  colnames(pf1temp) = c("week","storeid",paste("price",i,sep=""),paste("major",i,sep=""),paste("line",i,sep=""),paste("display",i,sep=""))
  hhall3.1 = merge(hhall3.1,pf1temp,all.x=TRUE)
}

hhall3.2 = hhall3[hhall3$totunits == 0,]

for(i in 1:length(upckeep)) {
  pf1temp = pf1w[pf1w$UPC == upckeep[i],c("erimweek","SID","price","MAJOR","LINE","DISPLAY")]
  colnames(pf1temp) = c("week","storeid",paste("price",i,sep=""),paste("major",i,sep=""),paste("line",i,sep=""),paste("display",i,sep=""))
  hhall3.2 = merge(hhall3.2,pf1temp,all.x=TRUE)
}

hhall3.3 = rbind(hhall3.1,hhall3.2)

hhall3.3 = hhall3.3[order(hhall3.3$hhid,hhall3.3$week),]

# make final estimation data for fortran

nvar = 10+3*length(upckeep)
nhhs = length(unique(hhall3.3$hhid))
nweek = 51
initweek = 87

prweights = rbind(c(0.1,0.32)/0.42,c(0.09,0.37)/0.46)   # these are the price averaging weights I used in the fortran code
# UPCs are 5: 79100002800, 6: 79100002450
# 10: 17800000240, 11: 17800000120

hhall4 = data.frame(kronecker(unique(hhall3.3$hhid),rep(1,nweek+initweek)),rep(allweek,nhhs))
colnames(hhall4) = c("hhid","week")
hhall4 = merge(hhall4,hhall3.3,all.x=TRUE)
hhall4$upc[is.na(hhall4$upc)] = 0
hhall4$totunits[is.na(hhall4$totunits)] = 0
hhall4$storeid[is.na(hhall4$storeid)] = 0  # in the fortran code no store visit is coded as zero
for(i in 1:length(upckeep)) {
  hhall4[[paste("price",i,sep="")]][is.na(hhall4[[paste("price",i,sep="")]])] = 0
  hhall4[[paste("major",i,sep="")]][is.na(hhall4[[paste("major",i,sep="")]])] = 0
  hhall4[[paste("line",i,sep="")]][is.na(hhall4[[paste("line",i,sep="")]])] = 0
  hhall4[[paste("display",i,sep="")]][is.na(hhall4[[paste("display",i,sep="")]])] = 0
}

hhall4 = hhall4[order(hhall4$hhid,hhall4$week),]

for(i in 1:3) {
  hhall4[[paste("bpurch",i,sep="")]][is.na(hhall4[[paste("bpurch",i,sep="")]])] = 0
  hhall4[[paste("units",i,sep="")]][is.na(hhall4[[paste("units",i,sep="")]])] = 0
}

# check for 0 prices when purchases happen - these are problematic
for(i in 1:length(upckeep)) {
  nbad = sum(hhall4$upc == upckeep[i] & hhall4[[paste("price",i,sep="")]] == 0)
  if(nbad > 0) {
    cat("error in prices:\n")
    cat(nbad,"observations of 0 price for upc",c(i,upckeep[i]),"\n")
  }
}

hhweek = array(0,c(nvar,initweek+nweek,nhhs))

# average prices for store visits where all prices are missing

mkt1stids = unique(hhall4$storeid[hhall4$storeid > 0])

avgprices = array(0,c(50,length(upckeep)))
badstids = NULL
for(i in 1:50) {
  if(sum(pf1w$SID == i) == 0 & i %in% mkt1stids) {
    badstids = c(badstids,i)
    # here we have a storeid that is visited sufficiently few times we don't really get a price - so just use average
    for(j in 1:length(upckeep)) {
      inds = hhpurch$upc == upckeep[j] & hhpurch$storeid == i
      if(sum(inds) > 0) {
        avgprices[i,j] = mean(hhpurch$price[inds]*100/hhpurch$units[inds],na.rm=TRUE)
      }
    }
  }
}


for(i in 1:(nweek+initweek)) {
  xx = hhall4$week == allweek[i]
  for(k in 1:3) {
      hhweek[2*k-1,i,] = hhall4[[paste("bpurch",k,sep="")]][xx]
      hhweek[2*k,i,] = hhall4[[paste("units",k,sep="")]][xx]/100
  }
  hhweek[7,i,] = hhall4$storeid[xx]

  for(j in 1:length(upckeep)) {
      hhweek[7+j,i,] = hhall4[[paste("price",j,sep="")]][xx]*100
      # fill in the missing prices
      for(st in badstids) {
        inds = hhall4$storeid[xx] == st
        if(sum(inds) > 0) {
          hhweek[7+j,i,inds] = avgprices[st,j]
        }
      }
      hhweek[7+length(upckeep)+j,i,] = pmax(hhall4[[paste("major",j,sep="")]][xx],hhall4[[paste("line",j,sep="")]][xx])
      hhweek[7+2*length(upckeep)+j,i,] = hhall4[[paste("display",j,sep="")]][xx]
  }
}


# store visit probability

stvisitprob = aggregate(hhall4$storeid[hhall4$week >= minweek] > 0,by=list(hhall4$hhid[hhall4$week >= minweek]), FUN=mean)

hhweek[nvar-2,,] = matrix(stvisitprob$x,nrow=nweek+initweek,ncol=nhhs,byrow=TRUE)

initweekdata = hhweek[,1:initweek,]
hhweek1 = hhweek[,1:nweek+initweek,]



sasprices = array(0,c(50,138,3*length(upckeep)))
deal = array(FALSE,c(50,138,length(upckeep)))
regprices = array(0,c(50,138,length(upckeep)))

write.table(as.integer(c(nvar,nhhs)),paste(outputpath,product,"/Rinfo_hn.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)
for(i in 1:nhhs) {
    write.table(t(hhweek1[,,i]),paste(outputpath,product,"/Rhhweek_hn.txt",sep=""),append = i > 1,quote=FALSE,row.names=FALSE,col.names=FALSE)
}

for(i in 1:nhhs) {
    write.table(t(initweekdata[,,i]),paste(outputpath,product,"/Rinitweek_hn.txt",sep=""),append = i > 1,quote=FALSE,row.names=FALSE,col.names=FALSE)
}

dummyarray = matrix(0,nrow=length(allweek),ncol=3*length(upckeep))
prarray = data.frame(allweek,stringsAsFactors=FALSE)
colnames(prarray) = "erimweek"

pf1w$feat = pmax(pf1w$MAJOR,pf1w$LINE)
for(i in 1:50) {
    if(sum(pf1w$SID == i) == 0 & i %in% mkt1stids) {
      # here we have a storeid that is visited sufficiently few times we don't really get a price - so just use average
        temp = prarray
        for(j in 1:length(upckeep)) {
            inds = hhpurch$upc == upckeep[j] & hhpurch$storeid == i
            if(sum(inds) > 0) {
                temp[[paste("price",j,sep="")]] = mean(hhpurch$price[inds]*100/hhpurch$units[inds],na.rm=TRUE)
            } else {
                temp[[paste("price",j,sep="")]] = 0
            }
        }
        for(j in 1:(2*length(upckeep))) {
            temp = cbind(temp,rep(0,nrow(temp)))
        }
        tempreg = temp
        tempdeal = temp
        tempdeal[,2:ncol(tempdeal)] = FALSE
        cat("using average prices for store ",i,"\n")
        write.table(temp[,1+1:(3*length(upckeep))],paste(outputpath,product,"/Rstores_hn.txt",sep=""),append = i > 1,quote=FALSE,row.names=FALSE,col.names=FALSE)
        sasprices[i,,] = as.matrix(temp[,1+1:(3*length(upckeep))])
        regprices[i,,] = as.matrix(temp[,1+1:(length(upckeep))])
    } else if(sum(pf1w$SID == i) > 0) {
        temp = prarray
        tempreg = prarray
        tempdeal = prarray
        for(j in 1:length(upckeep)) {
            temp = merge(temp,pf1w[pf1w$UPC == upckeep[j] & pf1w$SID == i,c("erimweek","price")],all.x=TRUE)
            colnames(temp)[j+1] = paste("price",j,sep="")
            temp[[paste("price",j,sep="")]][is.na(temp[[paste("price",j,sep="")]])] = 0
            temp[[paste("price",j,sep="")]] = temp[[paste("price",j,sep="")]]*100

            tempreg = merge(tempreg,pf1w[pf1w$UPC == upckeep[j] & pf1w$SID == i,c("erimweek","REG")],all.x=TRUE)
            colnames(tempreg)[j+1] = paste("REG",j,sep="")
            tempreg[[paste("REG",j,sep="")]][is.na(tempreg[[paste("REG",j,sep="")]])] = 0
            tempreg[[paste("REG",j,sep="")]] = tempreg[[paste("REG",j,sep="")]]*100

            tempdeal = merge(tempdeal,pf1w[pf1w$UPC == upckeep[j] & pf1w$SID == i,c("erimweek","sale")],all.x=TRUE)
            colnames(tempdeal)[j+1] = paste("sale",j,sep="")
            tempdeal[[paste("sale",j,sep="")]][is.na(tempdeal[[paste("sale",j,sep="")]])] = 0
            tempdeal[[paste("sale",j,sep="")]] = tempdeal[[paste("sale",j,sep="")]]*100
        }
        for(j in 1:length(upckeep)) {
            temp = merge(temp,pf1w[pf1w$UPC == upckeep[j] & pf1w$SID == i,c("erimweek","feat")],all.x=TRUE)
            colnames(temp)[j+1+length(upckeep)] = paste("feat",j,sep="")
            temp[[paste("feat",j,sep="")]][is.na(temp[[paste("feat",j,sep="")]])] = 0
        }
        for(j in 1:length(upckeep)) {
            temp = merge(temp,pf1w[pf1w$UPC == upckeep[j] & pf1w$SID == i,c("erimweek","DISPLAY")],all.x=TRUE)
            colnames(temp)[j+1+2*length(upckeep)] = paste("display",j,sep="")
            temp[[paste("display",j,sep="")]][is.na(temp[[paste("display",j,sep="")]])] = 0
        }
        temp = temp[order(temp$erimweek),]
        tempreg = tempreg[order(tempreg$erimweek),]
        tempdeal = tempdeal[order(tempdeal$erimweek),]
        psum = apply(temp[,1+1:length(upckeep)],1,sum)
        if(sum(psum == 0) > 0) {
            cat("imputing prices for store",i,",",sum(psum == 0),"weeks \n")
        #if it is at the beginning or end just impute forwards or backwards
            if(max(diff(which(psum==0))) == 1) {
                if(min(which(psum==0)) == 1) {
                    lastprice = temp[min(which(psum>0)),1+1:length(upckeep)]
                } else {
                    lastprice = temp[max(which(psum>0)),1+1:length(upckeep)]
                }
                temp[psum==0,1+1:length(upckeep)] = matrix(unlist(lastprice),nrow=sum(psum==0),ncol=length(upckeep),byrow=TRUE)
                tempreg[psum==0,1+1:length(upckeep)] = matrix(unlist(lastprice),nrow=sum(psum==0),ncol=length(upckeep),byrow=TRUE)
            } else {
          # break into pieces
                nbreak = 1+sum(diff(which(psum==0))>1)
                if(nbreak == 2 & min(which(psum==0)) == 1 & max(which(psum==0))==length(psum)) {
                    breakpoint = 1+which(diff(which(psum==0))>1)
                    lastp1 = temp[min(which(psum>0)),1+1:length(upckeep)]
                    lastp2 = temp[max(which(psum>0)),1+1:length(upckeep)]
                    temp[1:(breakpoint-1),1+1:length(upckeep)] = matrix(unlist(lastp1),nrow=breakpoint-1,ncol=length(upckeep),byrow=TRUE)
                    temp[(breakpoint+max(diff(which(psum==0)))):length(psum),1+1:length(upckeep)] = matrix(unlist(lastp2),nrow=length(psum)-(breakpoint+max(diff(which(psum==0))))+1,ncol=length(upckeep),byrow=TRUE)
                    tempreg[1:(breakpoint-1),1+1:length(upckeep)] = matrix(unlist(lastp1),nrow=breakpoint-1,ncol=length(upckeep),byrow=TRUE)
                    tempreg[(breakpoint+max(diff(which(psum==0)))):length(psum),1+1:length(upckeep)] = matrix(unlist(lastp2),nrow=length(psum)-(breakpoint+max(diff(which(psum==0))))+1,ncol=length(upckeep),byrow=TRUE)
                } else {
                    stop("price break in the middle - need a different procedure\n")
                }
            }
        }
        write.table(temp[,1+1:(3*length(upckeep))],paste(outputpath,product,"/Rstores_hn.txt",sep=""),append = i > 1,quote=FALSE,row.names=FALSE,col.names=FALSE)
        sasprices[i,,] = as.matrix(temp[,1+1:(3*length(upckeep))])
        regprices[i,,] = as.matrix(tempreg[,1+1:(length(upckeep))])
        deal[i,,] = as.matrix(tempdeal[,1+1:(length(upckeep))])
    } else {
        write.table(dummyarray,paste(outputpath,product,"/Rstores_hn.txt",sep=""),append = i > 1,quote=FALSE,row.names=FALSE,col.names=FALSE)
    }
}

niv = 4

# create the choice set for estimation

alphas = list( rbind(cratios[2,]/sum(cratios[2,]),cratios[3,]/sum(cratios[3,]),
                     cratios[4,]/sum(cratios[4,])) )

dyn.load("soup.dll")

csout = .Call("makecset",length(upckeep),alphas,2)

dyn.unload("soup.dll")

xbig = c(rep(0,length(upckeep)),8,0,0)
xstart = xbig

dyn.load("soup.dll")

# compute importance distribution
# estimate the brand level model and then use average moments of that
# we will normalize the first product to 1

ll.brand <- function(x1,hhweek1,csout,nupcs,alphas,fx,nchoices=3,niv=5) {

  x = rep(0,length(fx))
  x[!fx] = x1

  cat("parameter:",x,"\n")

  ll = .Call("llbrand",x,hhweek1,csout,nupcs,alphas,FALSE,FALSE,nchoices,niv)

  if(x[1+length(upckeep)] < 0) {
    penalty = 1000*(x[1+length(upckeep)])^2
  } else {
    penalty = 0
  }

  cat("log-likelihood:",sum(ll),"\n")

  return(-sum(ll)+penalty)

}


fx = rep(FALSE,length(xbig))
fx[1] = TRUE
xstart = xstart[-1]

test = ll.brand(xstart,hhweek1,csout,length(upckeep),alphas,fx,2,niv)

out = optim(xstart,ll.brand,control=list(maxit=25000),hhweek1=hhweek1,
            csout=csout,nupcs=length(upckeep),alphas=alphas,fx=fx,nchoices=2,niv=niv)

dyn.unload("soup.dll")

save(hhall4,hhpurch,sasprices,regprices,deal,hhweek,initweek,hhweek1,initweekdata,fmla1,fmla2,upckeep,cratios,out,file=paste(outputpath,product,"/sssoup_intermediatedata.RData",sep=""))
write.table(c(0,out$par),paste(outputpath,product,"/brandests.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

# compute inclusive values

dyn.load("soup.dll")

getiv <- function(x1,hhweek1,csout,nupcs,alphas,fx,nchoices=3,niv=5) {

  x = rep(0,length(fx))
  x[!fx] = x1

  iv = .Call("llbrand",x,hhweek1,csout,nupcs,alphas,TRUE,FALSE,nchoices,niv)[[1]]

  return(iv)

}

iv = getiv(out$par,hhweek1,csout,length(upckeep),alphas,fx,2,niv)

dyn.unload("soup.dll")


ivstats = vector("list",length=niv)

for(i in 1:niv) {

  ivvec = as.vector(iv[,,i])
  ivvec = ivvec[ivvec != 0]

  ivstats[[i]] = c(mean(ivvec),sd(ivvec))

}

# do mcmc to get parameter heterogeneity

hhids = unique(hhall4$hhid)
hhids = hhids[order(hhids)]

source("mcmc_soup.r")

write.table(ivstats,paste(outputpath,product,"/ivstats.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

write.table(ivmeanhh[,paste("iv",1:niv,sep="")],paste(outputpath,product,"/ivstatshh.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

write.table(ivsdhh[,paste("iv",1:niv,sep="")],paste(outputpath,product,"/ivstatshh.txt",sep=""),append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE)

dcsout = rep(0,niv)

dcsout[1] = dim(csout[[1]][[1]])[1]-1
for(i in 1:length(csout[[1]][[2]])) {
  dcsout[1+i] = dim(csout[[1]][[2]][[i]])[1]
}

write.table(dcsout,paste(outputpath,product,"/csetdims.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

csetnels = c(1,2,2,2)

write.table(csetnels,paste(outputpath,product,"/csetdims.txt",sep=""),append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE)

write.table(csout[[1]][[1]][-1,],paste(outputpath,product,"/choicesetout.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

for(i in 1:length(csout[[1]][[2]])) {
  write.table(csout[[1]][[2]][[i]],paste(outputpath,product,"/choicesetout.txt",sep=""),append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE)
}

write.table(csout[[2]][[1]][-1,],paste(outputpath,product,"/alphabigout.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

for(i in 1:length(csout[[1]][[2]])) {
  write.table(csout[[2]][[2]][[i]],paste(outputpath,product,"/alphabigout.txt",sep=""),append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE)
}

lalpha = c(1,3)

write.table(lalpha,paste(outputpath,product,"/lalpha.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

alphas1 = list(matrix(c(1,0,0),nrow=1,ncol=3),alphas[[1]])

for(i in 1:2) {
    write.table(alphas1[[i]],paste(outputpath,product,"/alphasmallout.txt",sep=""),append=(i>1),quote=FALSE,row.names=FALSE,col.names=FALSE)
}

pnames = rep("",length(upckeep))

j=0
for(i in upckeep) {
  j=j+1
  pnames[j] = substr(pf1w$desc[pf1w$UPC == i][1],1,20)
}

write.table(pnames,paste(outputpath,product,"/upcnames.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

