# Simulation study to show how the price series drives
# which index works better

rm(list=ls())

mpiflag = FALSE

library(markovchain)

path = "C:/Users/MattOsborne/Dropbox/price index paper/code/clean code for paper/Simulation Code/"  #location of code
figstub = "figuresandtables"  # supplementary figures (purchase hazards, etc) go here

tablepath = "C:/Users/MattOsborne/Dropbox/price index paper/output tables and figures/"    #tables for paper

# which price regime to simulate
# 1: analog to tuna category
# 2: analog to the soup category
pregime = 1


docoli = TRUE        #present COLI rather than price indexes
modalprices = FALSE  #use modal prices rather than average in the FS index
ccd = TRUE           #use the Caves-Christensen-Diewert formulation rather than FS

fform = as.integer(3)
npar = as.integer(8)

ntypes = as.integer(2)
gamm = 1
curv = -0.25
i0 = 0.0
i1 = -0.01
alph = 1
cc = -0.25
csmean = 1
csdelta = 0
coeff.reg = matrix(rep(c(gamm,curv,i0,i1,alph,cc,csmean,csdelta),ntypes),nrow=npar,ncol=ntypes)
mtype = 1
if(ntypes == 2) {
  coeff.reg[3:4,2] = 0
  mtype = c(0.75,0.25)
}


setwd(path)

repeatedgrid = TRUE

# exogenous or endogenous consumption model
modeltype = "endog"

# time period to start when analyzing moments
tstart = 51

# do we simulate future profits
simvf = TRUE


numprice = 2

if(pregime == 1) {
  # initial price regime
  pstates = c(0.5,2)
  pprobs = matrix(c(0.1,0.3,0.9,0.7),nrow=2,ncol=2)

  # second price regime
  pstates2 = c(0.5,1.5)
  pprobs2 = matrix(c(0.1,0.3,0.9,0.7),nrow=2,ncol=2)
} else if(pregime == 2) {
  # initial price regime
  pstates = c(0.5,10)
  pprobs = matrix(c(0.01,0.10,0.99,0.90),nrow=2,ncol=2)

  # second price regime
  pstates2 = c(0.4,10.1)
  pprobs2 = matrix(c(0.01,0.04,0.99,0.96),nrow=2,ncol=2)
}

cost = 0.6

delta = 0.95
deltafirm = 0.95

logitsd = 1

ibar = as.integer(11)

dyn.load("simulation_functions.dll")

source("simulation_functions.r")

bigrep = 1

coeff = coeff.reg

set.seed(8371363)

nt1 = 150
nt2 = 100
nhhs = 10000
nstore = 100
hhtypes = as.integer(sample(1:ntypes,nhhs,TRUE,mtype))

sp1 <- simprice(pstates,pprobs[,1:(length(pstates)-1)],nstore,nhhs,nt1)
simprices = sp1[[1]]
hhassign = sp1[[2]]

sp2 <- simprice(pstates2,pprobs2[,1:(length(pstates)-1)],nstore,
                nhhs,nt2,hhassign=hhassign,initprice=simprices$i[,nt1])
simprices2 = sp2[[1]]
hhassign2 = sp2[[2]]

cdraws.u = vector("list",length=ntypes)

cdraws = matrix(0.0,nrow=nhhs,ncol=nt1)

if(modeltype == "exog") {
  for(i in 1:ntypes) {
    cdraws.u[[i]] = matrix(runif(sum(hhtypes==i)*nt1),nrow=sum(hhtypes==i),ncol=nt)
    cdraws[hhtypes==i,] = as.numeric(cdraws.u[[i]] < coeff[7,i])
  }
}

istart = rep(0,nhhs)

info = list(ntypes=ntypes,nhhs=nhhs,nt=nt1,hhtypes=hhtypes,delta=delta,ibar=ibar)

simhh <- createdata(coeff,pstates,ntypes,delta,ibar,pprobs,cdraws,simprices,fform,npar,logitsd=logitsd,info=info)

info$nt = nt2

simhh2 <- createdata(coeff,pstates2,ntypes,delta,ibar,pprobs2,cdraws,simprices2,fform,npar,logitsd,istart = simhh$inv[,nt1],info=info)

# overall averages

agx = apply(simhh$x,2,mean)
aginv = apply(simhh$inv,2,mean)
agcons = apply(simhh$cons,2,mean)
agu = apply(simhh$util,2,mean)
agprice = apply(simhh$x*simprices$p,2,mean)
ucons = apply(simhh$util+simhh$x*simprices$p,2,mean)
fpriceu = apply(simhh$x*simprices$p/(simhh$util),2,mean)

agx2 = apply(simhh2$x,2,mean)
aginv2 = apply(simhh2$inv,2,mean)
agcons2 = apply(simhh2$cons,2,mean)
agu2 = apply(simhh2$util,2,mean)
agprice2 = apply(simhh2$x*simprices2$p,2,mean)
ucons2 = apply(simhh2$util+simhh2$x*simprices2$p,2,mean)
fpriceu2 = apply(simhh2$x*simprices2$p/(simhh2$util),2,mean)

aginv.t = vector("list",length=2)
agu.t = vector("list",length=2)
aginv2.t = vector("list",length=2)
agu2.t = vector("list",length=2)

for(i in 1:ntypes) {
  aginv.t[[i]] = apply(simhh$inv[hhtypes==i,],2,mean)
  aginv2.t[[i]] = apply(simhh2$inv[hhtypes==i,],2,mean)
  agu.t[[i]] = apply(simhh$util[hhtypes==i,],2,mean)
  agu2.t[[i]] = apply(simhh2$util[hhtypes==i,],2,mean)
}

# make all the different indexes
# remove first 50 periods for burn in

basepd = tstart

p0 = mean(simprices$p[,basepd])
p0base = p0
u0 = agu[basepd]/alph
ubig = c(agu,agu2)/alph
udiff = ubig - u0
if(docoli) {
  pdynamic = udiff
} else {
  pdynamic = 100*(p0-udiff)/p0
}

# Feenstra-Shapiro COLI

bigT = 50

xbig = cbind(simhh$x,simhh2$x)
xbig = xbig[,basepd:ncol(xbig)]

pbig = cbind(simprices$p,simprices2$p)
pbig = pbig[,basepd:ncol(pbig)]

ntFS = ceiling(ncol(xbig)/bigT)


piFS = rep(0,ncol(xbig))

if(ccd) {

  q0 = matrix(0,nrow=nstore,ncol=bigT)
  p0 = matrix(0,nrow=nstore,ncol=bigT)

  for(i in 1:nstore) {
    q0[i,] = apply(xbig[hhassign==i,1:bigT],2,sum)
    p0[i,] = apply(pbig[hhassign==i,1:bigT],2,min)
  }

  s0 = p0*q0/sum(p0*q0)

  avp0 = p0

} else {

  if(modalprices) {

    q0 = rep(0,nstore)
    p0 = rep(0,nstore)

    for(i in 1:nstore) {
      pp = apply(pbig[hhassign==i,1:bigT],2,min)
      ptab = table(pp)
      p0[i] = min(as.integer(names(ptab)[which.max(ptab)]))
      q0[i] = sum(xbig[hhassign==i,1:bigT]*(simprices$p[hhassign==i,1:bigT] == p0[i]))

    }

    avp0 = p0

    s0 = p0*q0/sum(p0*q0)

  } else {

    q0 = matrix(0,nrow=nstore,ncol=bigT)
    p0 = matrix(0,nrow=nstore,ncol=bigT)

    for(i in 1:nstore) {
      q0[i,] = apply(xbig[hhassign==i,1:bigT],2,sum)
      p0[i,] = apply(pbig[hhassign==i,1:bigT],2,min)
    }

    s0 = apply(p0*q0,1,sum)
    s0 = s0/sum(s0)

    avp0 = apply(p0*q0,1,sum)/apply(q0,1,sum)

  }

}


sik = vector("list",length=ntFS)

for(m in 1:ntFS) {
  scol = 1+(m-1)*bigT
  ecol = min(ncol(xbig),m*bigT)
  qik = matrix(0,nrow=nstore,ncol=ecol-scol+1)
  pik = matrix(0,nrow=nstore,ncol=ecol-scol+1)
  for(i in 1:nstore) {
    if(scol==ecol) {
      qik[i,] = sum(xbig[hhassign==i,scol])
      pik[i,] = min(pbig[hhassign==i,scol])
    } else {
      qik[i,] = apply(xbig[hhassign==i,scol:ecol],2,sum)
      pik[i,] = apply(pbig[hhassign==i,scol:ecol],2,min)
    }
  }
  sik[[m]] = pik*qik
  sik[[m]] = sik[[m]]/sum(sik[[m]])

  if(ccd) {
    piFS[scol:ecol] = exp(sum(0.5*(s0[,1:(ecol-scol+1)] + sik[[m]])*log(pik/avp0[,1:(ecol-scol+1)])))
  } else {
    piFS[scol:ecol] = exp(sum(0.5*(s0 + sik[[m]])*log(pik/avp0)))
  }
}

if(docoli) {
  piFS = p0base - piFS*p0base
} else {
  piFS = 100*piFS
}

pdynamic = pdynamic[basepd-1+1:ncol(xbig)]

# Chevalier Kashyap index

bestprice = rep(0,ncol(xbig))
stwts = rep(0,nstore)
for(i in 1:nstore) {
  stwts[i] = sum(xbig[hhassign==i,1:bigT])
}
stwts = stwts/sum(stwts)

for(m in 1:ntFS) {
  scol = 1+(m-1)*bigT
  ecol = min(ncol(xbig),m*bigT)
  pik = rep(0,nstore)
  for(i in 1:nstore) {
    pik[i] = min(pbig[hhassign==i,scol:ecol])
  }
  bestprice[scol:ecol] = sum(stwts*pik)
}

splitpd = ncol(xbig)-nt2

bestprice = c(rep(min(pstates),splitpd),rep(min(pstates2),nt2))

bestprice = 100*bestprice/bestprice[1]


# Laspeyres Index

laspeyres = rep(0,ncol(xbig))


pbase = rep(0,nstore)
qbase = rep(0,nstore)
for(i in 1:nstore) {
  pik = apply(pbig[hhassign==i,1:bigT],2,min)
  qik = apply(xbig[hhassign==i,1:bigT],2,sum)
  pbase[i] = mean(pik)
  qbase[i] = sum(xbig[hhassign==i,1])
}

stwtbase = qbase/sum(qbase)

for(j in 1:ncol(xbig)) {

  pik = rep(0,nstore)
  for(i in 1:nstore) {
    pik[i] = min(pbig[hhassign==i,j])
  }

  laspeyres[j] = sum(stwts*pik/pbase)

}

laspeyres = 100*laspeyres

piCK = mtype[1]*laspeyres + mtype[2]*bestprice

if(docoli) {
  laspeyres = p0base-laspeyres*p0base/100
  bestprice = p0base-bestprice*p0base/100
  piCK = p0base-piCK*p0base/100
}

outtab = c("Frequency of Promotions","Average Inventory","Average Consumption","Average Expenditure/Utility","Dynamic Index","Laspeyres","FS","Best Price","CK")

pfreq = c(rep(sum(simprices$i==1)/(nhhs*nt1),nt1-basepd+1),
rep(sum(simprices2$i==1)/(nhhs*nt2),nt2))

invtot = c(aginv[basepd:nt1],aginv2)
constot = c(agcons[basepd:nt1],agcons2)
fpu2 = c(fpriceu[basepd:nt1],fpriceu2)

indx = list(pfreq,invtot,constot,fpu2,pdynamic,laspeyres,piFS,bestprice,piCK)

sigfig = 2

for(i in 1:length(indx)) {
  outtab[i] = paste(outtab[i]," & ",round(mean(indx[[i]][1:splitpd]),sigfig),
        " & ",round(mean(indx[[i]][(splitpd+1):ncol(xbig)]),sigfig),
        " & ",round(mean(indx[[i]][(splitpd+1):ncol(xbig)])-
                        mean(indx[[i]][1:splitpd]),sigfig)," \\\\",sep="")
}

outtab = outtab[c(2,3,5,6,7,9)]

write.table(outtab,file=paste(tablepath,"indexsimulation_",
                   pregime,".tex",sep=""),quote=FALSE,row.names=FALSE,
            col.names=FALSE)


