// Routines for fake data experiment //

#include<math.h>
#include<R_ext/PrtUtil.h>
#include<R.h>
#include<Rinternals.h>
#include "funcs.h"

// flow utility

double util(double * co,int typ,double inv, double x, double cons, double p, int fform, double cshock, int npar)
{
  double inext = (inv + x - cons);
  double u = 0;
  if(fform == 1) {
    u = co[0+npar*typ]*(cons + cshock + co[1+npar*typ]*(cons+cshock)*(cons+cshock)) - co[4+npar*typ]*p*x + co[2+npar*typ]*inext + co[3+npar*typ]*inext*inext;
  } else {
    u = co[0+npar*typ]*log(cons + cshock) - co[4+npar*typ]*p*x + co[2+npar*typ]*inv + co[3+npar*typ]*inv*inv;
  }
  if(x>0)
    u+=co[5+npar*typ];
  return(u);
}


// Evaluate consumer value function

double vfeval(double * co, int typ, int nt, double inv, double p, int pi, double delta, int im, double * pprobs, int np, double * vf, int fform, int npar)
{
  double vret = 0;
  int ncs = 2;
  if(co[7+npar*typ] == 0)
    ncs = 1;
  for(int n=0;n<ncs;n++) {
    double cshock=0;
    if(n==0) {
      cshock = co[6+npar*typ]-co[7+npar*typ];
    } else {
      cshock = co[6+npar*typ]+co[7+npar*typ];
    }
    double eu = 0;
    for(int x=0;x < im;x++) {
      double maxu = -1000000;
      int mincons = 0;
      if((int)inv+x > im-1)
        mincons = (int)inv+x-(im-1);
      for(int c=mincons;c < (int)inv+x+1;c++) {
        double u = util(co,typ,inv,(double)x,(double)c,p,fform,cshock,npar);
        for(int i2=0;i2 < np;i2++) {
          u += delta*pprobs[pi+np*i2]*vf[typ+nt*(i2 + np*((int)inv+x-c))];
        }
        if(u > maxu) {
          maxu = u;
        }
      }
      eu += exp(maxu);
    }
    vret += log(eu)/((double)ncs);
  }
  return(vret);
}


// compute optimal consumption

void optutil(int x, double * co, int typ, int nt, double inv, double p, int pi, double delta, int im, double * pprobs, int np, double * vf, int * copt, double * maxu,
             int fform, double cshock, int npar)
{
  *maxu = -1000000;
  int mincons = 0;
  if((int)inv+x > im-1)
    mincons = (int)inv+x-(im-1);
  for(int c=mincons;c < (int)inv+x+1;c++) {
    double u = util(co,typ,inv,(double)x,(double)c,p,fform,cshock,npar);
    for(int i2=0;i2 < np;i2++) {
      u += delta*pprobs[pi+np*i2]*vf[typ+nt*(i2 + np*((int)inv+x-c))];
    }
    if(u > *maxu) {
      *maxu = u;
      *copt = c;
    }
  }
}



// Solve for value function for a given type through value function iteration

SEXP vfiter(SEXP coefin, SEXP ntypes, SEXP deltacons, SEXP ibar, SEXP pstates, SEXP np, SEXP pprobs, SEXP tol, SEXP fform1, SEXP npar1)
{
  int nt = *INTEGER(ntypes), im = *INTEGER(ibar), pm = *INTEGER(np);
  double *co = REAL(coefin);
  double delta = *REAL(deltacons);
  double *pall = REAL(pstates);
  double *pprob1 = REAL(pprobs);
  double rtol = *REAL(tol);
  int fform = *INTEGER(fform1);
  int npar = *INTEGER(npar1);

  SEXP vfout;
  PROTECT(vfout = allocVector(REALSXP, nt*pm*im ));
  double *rvfout = REAL(vfout);

  for(int i=0;i<nt*pm*im;i++) {
    rvfout[i] = 0;
  }

  double diff = rtol+1;
  double vfnew[nt*pm*im];

  while(diff > rtol) {
    diff = 0;
    for(int ty=0;ty<nt;ty++) {
      for(int p=0;p<pm;p++) {
        for(int i=0;i<im;i++) {
          vfnew[ty+nt*(p+pm*i)] = vfeval(co,ty,nt,(double)i,pall[p],p,delta,im,pprob1,pm,rvfout,fform,npar);
          diff = dmax(diff,fabs(vfnew[ty+nt*(p+pm*i)]-rvfout[ty+nt*(p+pm*i)]));
        }
      }
    }
    for(int i=0;i<nt*pm*im;i++) {
      rvfout[i] = vfnew[i];
    }
  }

  UNPROTECT(1);
  return(vfout);

}




// simulate individual choices

SEXP hhchoicesim(SEXP coefin, SEXP ntypes, SEXP deltacons, SEXP ibar, SEXP pstates, SEXP np, SEXP pprobs, SEXP vf, SEXP nhh, SEXP ntime,
                    SEXP istart, SEXP simpi, SEXP udraws, SEXP hhtypes1, SEXP fform1, SEXP npar1)
{

  int nt = *INTEGER(ntypes), im = *INTEGER(ibar), pm = *INTEGER(np);
  int nhhs = *INTEGER(nhh), nti = *INTEGER(ntime);
  int *ist = INTEGER(istart);
  int *spi = INTEGER(simpi);
  double *co = REAL(coefin);
  double delta = *REAL(deltacons);
  double *pall = REAL(pstates);
  double *pprob1 = REAL(pprobs);
  double *rvf = REAL(vf);
  double *ru = REAL(udraws);
  int *hhtypes = INTEGER(hhtypes1);
  int fform = *INTEGER(fform1);
  int npar = *INTEGER(npar1);

  SEXP simdata, xdata, idata, cdata, fudata, ivdata;
  PROTECT(simdata = allocVector(VECSXP, 5 ));
  PROTECT(xdata = allocVector(REALSXP,nhhs*nti));
  PROTECT(idata = allocVector(REALSXP,nhhs*nti));
  PROTECT(cdata = allocVector(REALSXP,nhhs*nti));
  PROTECT(fudata = allocVector(REALSXP,nhhs*nti));
  PROTECT(ivdata = allocVector(REALSXP,nhhs*nti));
  double *rx = REAL(xdata);
  double *ri = REAL(idata);
  double *rc = REAL(cdata);
  double *rfu = REAL(fudata);
  double *riv = REAL(ivdata);

  int ty=0;  //change this later

  double px[im];
  int cx[im];

  double cumprob[2];

  int copt;
  double maxu;
  double cshock;

  GetRNGstate();

  for(int hh=0;hh<nhhs;hh++) {
    int inv = ist[hh];
    ty = hhtypes[hh]-1;
    for(int t=0;t<nti;t++){
      double sx = 0;
      double uu = unif_rand();
      if(uu < 0.5) {
        cshock = co[6+npar*ty]-co[7+npar*ty];
      } else {
        cshock = co[6+npar*ty]+co[7+npar*ty];
      }
      rfu[hh+nhhs*t] = 0.0;
      riv[hh+nhhs*t] = 0.0;
      for(int x=0;x<im;x++) {
        optutil(x,co,ty,nt,(double)inv,pall[spi[hh+nhhs*t]-1],spi[hh+nhhs*t]-1,delta,im,pprob1,pm,rvf,&copt,&maxu,fform,cshock,npar);
        px[x] = exp(maxu);
        sx += px[x];
        cx[x] = copt;
        double uu = util(co,ty,(double)inv,(double)x,copt,
                         pall[spi[hh+nhhs*t]-1],fform,cshock,npar);
        rfu[hh+nhhs*t] += exp(uu);
        if(x > 0) {
          riv[hh+nhhs*t] += exp(-co[4+npar*ty]*pall[spi[hh+nhhs*t]-1]*((double)x));
        }
      }
      rfu[hh+nhhs*t] = log(rfu[hh+nhhs*t]);
      riv[hh+nhhs*t] = log(riv[hh+nhhs*t]);
      cumprob[0]=0;
      cumprob[1]=0;
      for(int x=0;x<im;x++) {
        cumprob[1]+=px[x]/sx;
        if(ru[hh+nhhs*t] >= cumprob[0] && ru[hh+nhhs*t] < cumprob[1]) {
          rx[hh+nhhs*t] = (double)x;
          ri[hh+nhhs*t] = (double)inv;
          rc[hh+nhhs*t] = (double)cx[x];
        }
        cumprob[0]=cumprob[1];
      }

      inv += (int)rx[hh+nhhs*t] - (int)rc[hh+nhhs*t];
    }
  }
  PutRNGstate();

  SET_VECTOR_ELT(simdata,0,xdata);
  SET_VECTOR_ELT(simdata,1,idata);
  SET_VECTOR_ELT(simdata,2,cdata);
  SET_VECTOR_ELT(simdata,3,fudata);
  SET_VECTOR_ELT(simdata,4,ivdata);
  UNPROTECT(6);
  return(simdata);

}











